/**************************************************************************//**
 * @file      : NUC1xx_i2c.c
 * @brief     : NUC1xx I2C Master Peripheral Interface
 * @version   : V1.0
 * @date      : 30. Jun. 2010
 * @author    : CooCox
******************************************************************************/
#include "stdio.h"
#include "nuc1xx.h"
#include "cox_types.h"
#include "cox_i2c.h"
/* I2C Control Clear register description */
/** Assert acknowledge Clear bit */
#define I2C_I2CONCLR_AAC                ((1<<2))
/** I2C interrupt Clear bit */
#define I2C_I2CONCLR_SIC                ((1<<3))
/** START flag Clear bit */
#define I2C_I2CONCLR_STAC               ((1<<5))
/** I2C interface Disable bit */
#define I2C_I2CONCLR_I2ENC              ((1<<6))
#define I2C_STAT_CODE_BITMASK           ((0xF8))

/* I2C Status Code definition (I2C Status register) */
#define I2C_I2CONSET_AA         ((0x04))   /* Assert acknowledge flag */
#define I2C_I2CONSET_SI         ((0x08))   /* I2C interrupt flag */
#define I2C_I2CONSET_STO        ((0x10))   /* STOP flag */
#define I2C_I2CONSET_STA        ((0x20))   /* START flag */
#define I2C_I2CONSET_I2EN       ((0x40))   /* I2C interface enable */

/* Master transmit mode -------------------------------------------- */
/** A start condition has been transmitted */
#define I2C_I2STAT_M_TX_START           ((0x08))
/** A repeat start condition has been transmitted */
#define I2C_I2STAT_M_TX_RESTART         ((0x10))
/** SLA+W has been transmitted, ACK has been received */
#define I2C_I2STAT_M_TX_SLAW_ACK        ((0x18))
/** SLA+W has been transmitted, NACK has been received */
#define I2C_I2STAT_M_TX_SLAW_NACK       ((0x20))
/** Data has been transmitted, ACK has been received */
#define I2C_I2STAT_M_TX_DAT_ACK         ((0x28))
/** Data has been transmitted, NACK has been received */
#define I2C_I2STAT_M_TX_DAT_NACK        ((0x30))
/** Arbitration lost in SLA+R/W or Data bytes */
#define I2C_I2STAT_M_TX_ARB_LOST        ((0x38))

/* Master receive mode -------------------------------------------- */
/** A start condition has been transmitted */
#define I2C_I2STAT_M_RX_START           ((0x08))
/** A repeat start condition has been transmitted */
#define I2C_I2STAT_M_RX_RESTART         ((0x10))
/** Arbitration lost */
#define I2C_I2STAT_M_RX_ARB_LOST        ((0x38))
/** SLA+R has been transmitted, ACK has been received */
#define I2C_I2STAT_M_RX_SLAR_ACK        ((0x40))
/** SLA+R has been transmitted, NACK has been received */
#define I2C_I2STAT_M_RX_SLAR_NACK       ((0x48))
/** Data has been received, ACK has been returned */
#define I2C_I2STAT_M_RX_DAT_ACK         ((0x50))
/** Data has been received, NACK has been return */
#define I2C_I2STAT_M_RX_DAT_NACK        ((0x58))

/***************************************************************************//**
 * @brief     Generate a start condition on I2C bus     
 * @param[in] I2Cx : I2C peripheral selected 
 * @return    value of I2C status register after generate a start condition
*******************************************************************************/
static uint32_t NUC1xx_I2C_Start (I2C_T *I2Cx)
{
  REG32WRITE(I2Cx->CON,SI,0);
  REG32WRITE(I2Cx->CON,STA,1);
  /* Wait for complete */
  while (I2Cx->CON.SI == 0);
  return (I2Cx->STATUS & I2C_STAT_CODE_BITMASK);
}


/***************************************************************************//**
 * @brief     Generate a stop condition on I2C bus    
 * @param[in] I2Cx : I2C peripheral selected 
 * @return    None
*******************************************************************************/
static void NUC1xx_I2C_Stop (I2C_T *I2Cx)
{
  /* Make sure start bit is not active */
  if (I2Cx->CON.STA == 1) {
	  REG32WRITE(I2Cx->CON,STA,0);
  }
  REG32WRITE(I2Cx->CON,STO,1);
  REG32WRITE(I2Cx->CON,SI,1);
  REG32WRITE(I2Cx->CON,AA,0);
}

/***************************************************************************//**
 * @brief     Send a byte    
 * @param[in] I2Cx : I2C peripheral selected 
 * @param[in] dat  : Data to send
 * @return    value of I2C status register after sending
*******************************************************************************/
static uint32_t NUC1xx_I2C_SendByte (I2C_T *I2Cx, uint8_t dat)
{
  /* Make sure start bit is not active */
  if (I2Cx->CON.STA == 1) {
	  REG32WRITE(I2Cx->CON,STA,0);
  }
	I2Cx->DATA = dat;
	if (I2Cx->STATUS == 0x08)
	{
	  REG32WRITE(I2Cx->CON,STO,0);
	  REG32WRITE(I2Cx->CON,SI,1);
	  REG32WRITE(I2Cx->CON,AA,0);
	}
	REG32WRITE(I2Cx->CON,AA,0);
	REG32WRITE(I2Cx->CON,EI,0);
	while (I2Cx->CON.SI == 0);
  return (I2Cx->STATUS & I2C_STAT_CODE_BITMASK);
}


/***************************************************************************//**
 * @brief     Get a byte    
 * @param[in] I2Cx : I2C peripheral selected 
 * @param[in] retdat : Pointer to read data
 * @param[in] ack : Response ACK or NACK
 * @return    value of I2C status register after receiving
*******************************************************************************/
static uint32_t NUC1xx_I2C_GetByte (I2C_T *I2Cx, uint8_t *dat, uint8_t ack)
{
  if (ack == 1) {
 	  REG32WRITE(I2Cx->CON,AA,1);
	  }
	  else {
 	  REG32WRITE(I2Cx->CON,AA,0);
  }
  REG32WRITE(I2Cx->CON,SI,0);   
  while (I2Cx->CON.SI==0);
	*dat = (uint8_t) (I2Cx->DATA & 0xFF);
  return (I2Cx->STATUS & I2C_STAT_CODE_BITMASK);
}

/***************************************************************************//**
 * @brief     Setup clock rate for I2C peripheral	      
 * @param[in] I2Cx : I2C peripheral selected, should be NUC1xx_I2C
 * @param[in] target_clock : clock of SSP (Hz)
 * @return    The read data
*******************************************************************************/
static uint32_t NUC1xx_I2C_SetClock (I2C_T *I2Cx, uint32_t rate)
{
	uint32_t Hclk,u32Freqout,u32PLLSrc,u32Freq = 0,u32AHBDivider;
	uint32_t au32Map[4] = {1, 2, 2, 4};
	uint32_t div;

	/*         Get clock_Hz                                      */
	if (SYSCLK->CLKSEL0.HCLK_S == 0)		/* external 12MHz crystal clock */
	{
		u32Freqout = 12000;	
	}
	else if(SYSCLK->CLKSEL0.HCLK_S == 1)    /* external 32KHz crystal clock */ 
	{
		u32Freqout = 32;
	}
	else if(SYSCLK->CLKSEL0.HCLK_S == 2)	/* PLL clock */
	{
		if (SYSCLK->PLLCON.PLL_SRC == 1)
	  {
			u32PLLSrc = 22000;	
  	}
	else
	{
		u32PLLSrc = 12000;
	}
	if (SYSCLK->PLLCON.PD ==1)	
		u32Freq = 0;	
	else		
		u32Freq = u32PLLSrc * (SYSCLK->PLLCON.FB_DV+2) / (SYSCLK->PLLCON.IN_DV+2) / au32Map[SYSCLK->PLLCON.OUT_DV];
		u32Freqout = u32Freq;
	}
	else if(SYSCLK->CLKSEL0.HCLK_S == 3)	/* internal 10KHz oscillator clock */
	{
		u32Freqout = 10;
	}
	else									/* internal 22MHz oscillator clock */
	{
		u32Freqout = 22000;
	}
	u32AHBDivider = SYSCLK->CLKDIV.HCLK_N + 1 ;
	Hclk = (u32Freqout/u32AHBDivider) * 1000;

	div = (uint32_t) (((Hclk * 10)/(rate * 4) + 5) / 10 - 1);
	if (I2Cx == I2C0)
	{
		I2C0->CLK = div;
	}
	else
	{
		I2C1->CLK = div;
	}
	return div;
}


/***************************************************************************//**
 * @brief     Setup clock rate for I2C peripheral	      
 * @param[in] I2Cx : I2C peripheral selected, should be NUC_I2C
 * @param[in] target_clock : clock of SSP (Hz)
 * @return    Result, should be
 *              -COX_SUCCESS : No error occurred
 *              -COX_ERROR : Error occurred, parameter is not supported 
*******************************************************************************/
static COX_Status NUC1xx_I2C_Init (I2C_T *I2Cx, uint32_t rate) 
{
	if(I2Cx == I2C0)
	{
		REG32WRITE(SYSCLK->APBCLK,I2C0_EN,1);    /*I2C Clock Enable*/
		REG32WRITE(SYS->IPRSTC2,I2C0_RST,1);/*IP set*/
		REG32WRITE(SYS->IPRSTC2,I2C0_RST,0);
		REG32WRITE(I2C0->CON,ENSI,1);	 /*I2C control enable*/

    /* Set Pin */
		REG32WRITE(SYS->GPAMFP,I2C0_SDA,1);
		REG32WRITE(SYS->GPAMFP,I2C0_SCL,1);
  }
  else if(I2Cx == I2C1)
  {
		REG32WRITE(SYSCLK->APBCLK,I2C1_EN,1);    /*I2C Clock Enable*/
		REG32WRITE(SYS->IPRSTC2,I2C1_RST,1);/*IP set*/
		REG32WRITE(SYS->IPRSTC2,I2C1_RST,0);
		REG32WRITE(I2C1->CON,ENSI,1);	 /*I2C control enable*/

    /* Set Pin */
		REG32WRITE(SYS->GPAMFP,I2C1_SDA,1);
		REG32WRITE(SYS->GPAMFP,I2C1_SCL,1);
    }
	else
	{
		return   COX_ERROR;
	}

	/* Set clock rate */
	NUC1xx_I2C_SetClock(I2Cx,rate);

	/* Set I2C operation to default */
	REG32WRITE(I2Cx->CON,AA,0);
	REG32WRITE(I2Cx->CON,STA,0);
	REG32WRITE(I2Cx->CON,ENSI,0);

	/* Enable I2C Peripheral */  
	REG32WRITE(I2Cx->CON,ENSI,1);

	return COX_SUCCESS;

}

/***************************************************************************//**
 * @brief     Transmit and Receive data      
 * @param[in] I2Cx : I2C peripheral selected, should be NUC1xx_I2C
 * @param[in] cfg : Pointer to a I2C_Transfer_Cfg structure that contains 
 *                  specified information about the configuration for master 
 *                  transfer.
 * @return    Result, should be
 *              -COX_SUCCESS : No error occurred
 *              -COX_ERROR : Error occurred, parameter is not supported 
 * Note:
 * - In case of using I2C to transmit data only, either transmit length set to 0
 *   or transmit data pointer set to NULL.
 * - In case of using I2C to receive data only, either receive length set to 0
 *   or receive data pointer set to NULL.
 * - In case of using I2C to transmit followed by receive data, transmit length,
 *   transmit data pointer, receive length and receive data pointer should be set
 *   corresponding.
*******************************************************************************/
static COX_Status NUC1xx_I2C_Transfer (I2C_T *I2Cx, I2C_Transfer_Cfg *cfg) 
{
  uint8_t *txdat;
  uint8_t *rxdat;
  uint32_t CodeStatus;
	uint8_t tmp;
  /*  reset all default state	 */
  txdat = (uint8_t *) cfg->wbuf;
  rxdat = (uint8_t *) cfg->rbuf;
  /*  Reset I2C setup value to default state		*/
  cfg->wcnt = 0;
  cfg->rcnt = 0;
  CodeStatus = 0;
	REG32WRITE(I2Cx->CON,EI,0);
	/*  Start command			*/
  CodeStatus = NUC1xx_I2C_Start(I2Cx);
  if ((CodeStatus != I2C_I2STAT_M_TX_START) && \
      (CodeStatus != I2C_I2STAT_M_TX_RESTART)) {
		NUC1xx_I2C_Stop(I2Cx);
  	return COX_ERROR; 
  }

	/* In case of sending data first */
	if ((cfg->wlen != 0) && (cfg->wbuf != COX_NULL))
	{
		REG32WRITE(I2Cx->CON,EI,1);
		/* Send slave address + WR direction bit = 0 */
    CodeStatus = NUC1xx_I2C_SendByte(I2Cx, (cfg->slave << 1));
    if (CodeStatus != I2C_I2STAT_M_TX_SLAW_ACK) 
    {
			NUC1xx_I2C_Stop(I2Cx);
			return COX_ERROR; 
		}
		
		/* Send a number of data bytes */
		while (cfg->wcnt < cfg->wlen)
		{	
			REG32WRITE(I2Cx->CON,EI,1);
			CodeStatus = NUC1xx_I2C_SendByte(I2Cx, *txdat);
      if (CodeStatus != I2C_I2STAT_M_TX_DAT_ACK) 
      {
				NUC1xx_I2C_Stop(I2Cx);
				return COX_ERROR; 
      }
    	txdat++;
      cfg->wcnt++;
		}
	}

	/* Second Start condition (Repeat Start) */
	if ((cfg->wlen != 0) && (cfg->wbuf != COX_NULL) && (cfg->rlen != 0) && (cfg->rbuf != COX_NULL)) 
	{
		CodeStatus = NUC1xx_I2C_Start(I2Cx);
		if ((CodeStatus != I2C_I2STAT_M_RX_START) && (CodeStatus != I2C_I2STAT_M_RX_RESTART)) 
		{
			NUC1xx_I2C_Stop(I2Cx);
			return COX_ERROR; 
		}
	}

	/* Then, start reading after sending data */
	if ((cfg->rlen != 0) && (cfg->rbuf != COX_NULL))
	{
		/* Send slave address + RD direction bit = 1 */

		CodeStatus = NUC1xx_I2C_SendByte(I2Cx, ((cfg->slave << 1) | 0x01));
		if (CodeStatus != I2C_I2STAT_M_RX_SLAR_ACK) 
   	{
			NUC1xx_I2C_Stop(I2Cx);
			return COX_ERROR; 
    }

		/* Receive a number of data bytes */
		while (cfg->rcnt < cfg->rlen)
		{
      /*
      * Note that: if data length is only one, the master should not
      * issue an ACK signal on bus after reading to avoid of next data frame
      * on slave side
      */
      if (cfg->rcnt < (cfg->rlen - 1))
      {
      	/*  Issue an ACK signal for next data frame	 */
        CodeStatus = NUC1xx_I2C_GetByte(I2Cx, &tmp, 1);
        if (CodeStatus != I2C_I2STAT_M_RX_DAT_ACK) 
        {
					NUC1xx_I2C_Stop(I2Cx);
					return COX_ERROR; 
        }
      } 
      else 
      {
      	/*  Do not issue an ACK signal	  */
      	CodeStatus = NUC1xx_I2C_GetByte(I2Cx, &tmp, 0);
      	if (CodeStatus != I2C_I2STAT_M_RX_DAT_NACK)
        {
					NUC1xx_I2C_Stop(I2Cx);
					return COX_ERROR; 
        }
			}
      *rxdat++ = tmp;
      cfg->rcnt++;
		}
	}	
	/* Send STOP condition */
	NUC1xx_I2C_Stop(I2Cx);
	return COX_SUCCESS;
}


/***************************************************************************//**
 * @brief     Check whether the slave device is ok or not	      
 * @param[in] I2Cx : I2C peripheral selected, should be NUC1xx_I2C
 * @param[in] slave_adr : slave device selected
 * @return    Result, should be
 *              -COX_SUCCESS : Device is ok
 *              -COX_ERROR : Error occurred, bus error or device is not found
*******************************************************************************/
static COX_Status NUC1xx_I2C_CheckSlave (I2C_T *I2Cx, uint8_t slave_adr)
{
  uint32_t CodeStatus = 0;
  /* Start command */
	REG32WRITE(I2Cx->CON,EI,0);
  CodeStatus = NUC1xx_I2C_Start(I2Cx);	
  if ((CodeStatus != I2C_I2STAT_M_TX_START) && \
     (CodeStatus != I2C_I2STAT_M_TX_RESTART) &&
     (CodeStatus != I2C_I2STAT_M_RX_START) && 
     (CodeStatus != I2C_I2STAT_M_RX_RESTART)) {    
  		NUC1xx_I2C_Stop(I2Cx); 
  		return COX_ERROR;
  }
  /* Send slave address + WR direction bit = 0 */
  CodeStatus = NUC1xx_I2C_SendByte(I2Cx, (slave_adr << 1));	
  if (CodeStatus != I2C_I2STAT_M_TX_SLAW_ACK) {
  		NUC1xx_I2C_Stop(I2Cx); 
  		return COX_ERROR;
  }
  NUC1xx_I2C_Stop(I2Cx); 
  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Reset I2C bus, generate a start conditon followed by a stop condition
 * @param[in] I2Cx : I2C peripheral selected, should be LPC_I2C
 * @param[in] slave_adr : slave device selected
 * @return    Result, should be
 *              -COX_SUCCESS : Reset OK
 *              -COX_ERROR : Error occurred, bus error
*******************************************************************************/
static COX_Status NUC1xx_I2C_BusReset (I2C_T *I2Cx)
{
  uint32_t CodeStatus = 0;
  
  /* Start command */
  CodeStatus = NUC1xx_I2C_Start(I2Cx);
  if ((CodeStatus != I2C_I2STAT_M_TX_START) && \
      (CodeStatus != I2C_I2STAT_M_TX_RESTART) &&
      (CodeStatus != I2C_I2STAT_M_RX_START) && 
      (CodeStatus != I2C_I2STAT_M_RX_RESTART)) {
    return COX_ERROR;
  }

  NUC1xx_I2C_Stop(I2Cx);
  return COX_SUCCESS;
}


/***************************************************************************//**
 * @brief     Configure I2C peripheral
 * @param[in] I2Cx : I2C Peripheral selected
 * @param[in] index : Configuration option, should be
 *                    -COX_I2C_CFG_RATE : clock rate
 * @param[in] arg : Argument
 * @return    Result, should be
 *              -COX_ERROR : The option is not support 
 *              -COX_SUCCESS
*******************************************************************************/
static COX_Status NUC1xx_I2C_Cfg (I2C_T *I2Cx, uint8_t index, uint32_t arg, uint32_t *pre_arg)
{
	COX_Status result = COX_ERROR;
  uint32_t arg_bak;

  switch(index) {
  	case COX_I2C_CFG_RATE : 
    	arg_bak = NUC1xx_I2C_SetClock(I2Cx, arg); 
    	result = COX_SUCCESS;
    	break;
  	default : break;
	}

	if((result == COX_SUCCESS) && (pre_arg != COX_NULL)) {
		*pre_arg = arg_bak;  
  }
  return result;
}


static COX_Status  NUC1xx_I2C0_Init (uint32_t rate) { return  NUC1xx_I2C_Init(I2C0, rate); } 
static COX_Status  NUC1xx_I2C1_Init (uint32_t rate) { return  NUC1xx_I2C_Init(I2C1, rate); } 

static COX_Status  NUC1xx_I2C0_Transfer (I2C_Transfer_Cfg *cfg)  { return  NUC1xx_I2C_Transfer(I2C0, cfg); }
static COX_Status  NUC1xx_I2C1_Transfer (I2C_Transfer_Cfg *cfg)  { return  NUC1xx_I2C_Transfer(I2C1, cfg); }

static COX_Status  NUC1xx_I2C0_CheckSlave (uint8_t adr) { return  NUC1xx_I2C_CheckSlave(I2C0, adr); }
static COX_Status  NUC1xx_I2C1_CheckSlave (uint8_t adr) { return  NUC1xx_I2C_CheckSlave(I2C1, adr); }

static COX_Status  NUC1xx_I2C0_BusReset (void) { return  NUC1xx_I2C_BusReset(I2C0); }
static COX_Status  NUC1xx_I2C1_BusReset (void) { return  NUC1xx_I2C_BusReset(I2C1); }

static COX_Status  NUC1xx_I2C0_Cfg (uint8_t index, uint32_t arg, uint32_t *pre_arg) { return  NUC1xx_I2C_Cfg(I2C0, index, arg, pre_arg); }
static COX_Status  NUC1xx_I2C1_Cfg (uint8_t index, uint32_t arg, uint32_t *pre_arg) { return  NUC1xx_I2C_Cfg(I2C1, index, arg, pre_arg); }


/***************************************************************************//**
 * Define NUC1xx COX I2C Peripheral Interface based on I2C0
*******************************************************************************/
COX_I2C_PI pi_i2c0 = 
{
   NUC1xx_I2C0_Init,      
   NUC1xx_I2C0_Transfer,
   NUC1xx_I2C0_CheckSlave,
   NUC1xx_I2C0_BusReset,
   NUC1xx_I2C0_Cfg,  
};

/***************************************************************************//**
 * Define NUC1xx COX I2C Peripheral Interface based on I2C1
*******************************************************************************/
COX_I2C_PI pi_i2c1 = 
{
   NUC1xx_I2C1_Init,      
   NUC1xx_I2C1_Transfer,
   NUC1xx_I2C1_CheckSlave,
   NUC1xx_I2C1_BusReset,
   NUC1xx_I2C1_Cfg,  
};
