/*******************************************************************************
 * @file      : nuc_pio.c
 * @brief     : CoX PIO Peripheral Interface of NUC1xx
 * @version   : V1.0
 * @date      : 6 July. 2010
 * @author    : CooCox
*******************************************************************************/
#include "NUC1xx.H" 
#include "NUC1xx_PIO.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
#define PORT_OFFSET   0x40
#define OFFSET   0x04
/*****************************************************************************
 * @brief	    Setup the pin as PIO	 
 * @param[in] pio 
 * @return    Result
 *            - COX_SUCCESS : Success
 *            - COX_ERROR   : Failed
*****************************************************************************/
static COX_Status NUC_GPIO_Init(COX_PIO_Dev pio)
{
	uint8_t u8Port = COX_PIO_PORT(pio);
	uint8_t u8Pin  = COX_PIO_PIN(pio);
  
	if( (u8Port > 4) || (u8Pin > 15) ) 
	{
		return COX_ERROR; 
	}
  
	/* Set the Pin as a GPIO */
	outpw((uint32_t)&SYS->GPAMFP + u8Port * OFFSET,\
		inpw((uint32_t)&SYS->GPAMFP+ u8Port * OFFSET) & (~(1 << u8Pin)));
		  
	return COX_SUCCESS;
}


/*****************************************************************************
 * @brief		  Setup the PIO as input or output
 * @param[in] pio  
 * @param[in] dir
 *            -0: Input
 *            -1: Output
 * @return 
 *            - COX_SUCCESS : Success
 *            - COX_ERROR   : Failed   
*****************************************************************************/
static COX_Status NUC_GPIO_SetDir(COX_PIO_Dev pio, uint8_t dir)
{
	uint8_t u8Port = COX_PIO_PORT(pio);
	uint8_t u8Pin  = COX_PIO_PIN(pio);
	
	if( (u8Port > 4) || (u8Pin > 15) || (dir > 3)) 
	{
		return COX_ERROR; 
	}

 	switch(dir)
	{
		case 0:	//in
			outpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET,\
				inpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET) & (~(3 << (u8Pin * 2))));
			break;
		case 1: //out
			outpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET,\
				inpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET) & (~(3 << (u8Pin * 2))));
			outpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET,\
				inpw((uint32_t)&GPIOA->PMD + u8Port * PORT_OFFSET) | (1 << (u8Pin * 2)));
			break;
		default:		    
			break;	
	}
  	  					    
	return COX_SUCCESS; 
}

/*******************************************************************************
 * @brief	    Change the PIO state	 
 * @param[in] pio 
 * @param[in] level 
 * @return    Previous state
*******************************************************************************/
static uint8_t  NUC_GPIO_Out (COX_PIO_Dev pio, uint8_t level)
{
	uint8_t  u8OldLevel;
	uint8_t  u8Port = COX_PIO_PORT(pio);
	uint8_t  u8Pin  = COX_PIO_PIN(pio);
	uint32_t u32PortAddr;

	if( (u8Port > 4) || (u8Pin > 15) ) 
	{
		return 0; 
	}

	u32PortAddr = (uint32_t)&GPIOA->PIN  + u8Port * PORT_OFFSET;
	if( ( inpw(u32PortAddr)>> u8Pin) & 1 )
	{
		u8OldLevel = 1; /* High Level */
	} 
	else 
	{
		u8OldLevel = 0; /* Low Level */	  
	}
    
	u32PortAddr = (uint32_t)&GPIOA->DOUT  + u8Port * PORT_OFFSET;
	if(level == 0) 
	{
		outpw(u32PortAddr, inpw(u32PortAddr) & (~(1 << u8Pin))); /* Output Low Level */	   	
	} 
	else 
	{
		/* Output High Level */
		switch(u8Port)
		{
			case 0:
				GPIOA->DOUT |=	1 << u8Pin;
				break;
			case 1:
				GPIOB->DOUT |=	1 << u8Pin;
				break;
			case 2:
				GPIOC->DOUT |=	1 << u8Pin;
				break;
			case 3:
				GPIOD->DOUT |=	1 << u8Pin;
				break;
			case 4:
				GPIOE->DOUT |=	1 << u8Pin;
				break;
			default:
				break;	
		}		
	}

	return u8OldLevel; 
}

/*****************************************************************************
 * @brief	    Get the Pin state
 * @param[in] pio  
 * @return    
*****************************************************************************/
static uint8_t  NUC_GPIO_Read (COX_PIO_Dev pio)
{
	uint8_t u8Level;
	uint8_t u8Port = COX_PIO_PORT(pio);
	uint8_t u8Pin  = COX_PIO_PIN(pio);
	
	if( (u8Port > 4) || (u8Pin > 15) ) 
	{
		return 0; 
	}

	if( ( inpw((uint32_t)&GPIOA->PIN + u8Port * PORT_OFFSET)>> u8Pin) & 1 )
	{
		u8Level = 1; /* High Level */
	} 
	else 
	{
		u8Level = 0; /* Low Level */
	}
	
	return u8Level; 
}

/*****************************************************************************
 * @brief	    Configure PIO, currently not achieve 
 * @param[in] pio 
 * @param[in] index 
 * @param[in] arg
 * @return    Previous state
*****************************************************************************/
static COX_Status NUC_GPIO_Cfg (COX_PIO_Dev pio, uint8_t index, uint32_t arg, uint32_t *pre_arg)
{  
	return COX_ERROR; 
}

/*******************************************************************************
 * Define LPC17xx CoX PIO Peripheral Interface
*******************************************************************************/
COX_PIO_PI pi_pio = 
{
	NUC_GPIO_Init,
	NUC_GPIO_SetDir,
	NUC_GPIO_Out,
	NUC_GPIO_Read,
	NUC_GPIO_Cfg  
}; 
