/***************************************************************************//**
 * @file      : NUC1xx_rtc.h
 * @brief     : NUC1xx RTC Interface
 * @version   : V1.0
 * @date      : 28. Jul. 2010
 * @author    : CooCox
*******************************************************************************/
#include"stdio.h"
#include"NUC1xx_RTC.h"
#include"nuc1xx.h"
#include"cox_types.h"
#include"cox_rtc.h"

static uint32_t volatile g_u32hiYear,g_u32loYear,g_u32hiMonth,g_u32loMonth,g_u32hiDay,g_u32loDay;
static uint32_t volatile g_u32hiHour,g_u32loHour,g_u32hiMin,g_u32loMin,g_u32hiSec,g_u32loSec;

RTC_Event_Handler rtc_alarm_event = COX_NULL;
RTC_Event_Handler rtc_inc_event = COX_NULL;


/* Time Counter Group and Alarm register group ----------------------------- */
#define RTC_SEC_HI_MASK    (0x00000007)  /** SEC hi register mask   */
#define RTC_SEC_LO_MASK    (0x0000000F)  /** SEC lo register mask   */
#define RTC_MIN_HI_MASK    (0x00000007)  /** MIN hi register mask   */
#define RTC_MIN_LO_MASK    (0x0000000F)  /** MIN lo register mask   */
#define RTC_HOUR_HI_MASK   (0x00000003)  /** HOUR hi register mask  */
#define RTC_HOUR_LO_MASK   (0x0000000F)  /** HOUR hi register mask  */
#define RTC_DOW_MASK       (0x00000007)  /** DOW register mask   */
#define RTC_DAY_HI_MASK    (0x00000003)  /** DaY HI  register mask   */
#define RTC_DAY_LO_MASK    (0x0000000F)  /** DaY LO  register mask   */
#define RTC_MONTH_HI_MASK  (0x00000001)  /** MONTH HI register mask */
#define RTC_MONTH_LO_MASK  (0x0000000F)  /** MONTH LO register mask */
#define RTC_YEAR_HI_MASK   (0x0000000F)  /** YEAR HI register mask  */
#define RTC_YEAR_LO_MASK   (0x0000000F)  /** YEAR LO register mask  */

/*---------------------------------------------------------------------------------------------------------*/
/*  RTC Access Key 		                                                          						   */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVRTC_INIT_KEY			0xa5eb1357
#define DRVRTC_WRITE_KEY		0xA965

/*---------------------------------------------------------------------------------------------------------*/
/*  RTC Initial Time Out Value				                                                           	   */
/*---------------------------------------------------------------------------------------------------------*/
#define DRVRTC_WAIT_COUNT		0xFFFFFFFF

#define RTC_AIER_INT			((1<<1))
/** Counter Increment Interrupt bit for INT */
#define RTC_AIER_ALARM 			((1<<0))
/** Counter Increment Interrupt bit for ALARM  */

#define RTC_INT_ALARM                        ((uint32_t)1 << 0) /* The alarm interrupt */
#define RTC_INT_COUNTER_INCREASE            ((uint32_t)1 << 0)   /* Counter Increment Interrupt */

/***************************************************************************//**
 * @brief     RTC Handler
 * @param[in] None
 * @return    None
*******************************************************************************/
void RTC_IRQHandler(void)
{

	/* This is increment counter interrupt, Second Interrupt */
	if(RTC->RIIR.TI & (RTC_INT_COUNTER_INCREASE))
	{
		if(rtc_inc_event != COX_NULL)
		{
			rtc_inc_event();
		}
		//REG32WRITE(RTC->RIIR,TI,1);		        /* Clear Interrupt Pending Bit */
	}
	/* This is Alarm interrupt */
	if( RTC->RIIR.AI & RTC_INT_ALARM )
	{
		if(rtc_alarm_event != COX_NULL)
		{
			rtc_alarm_event();
		}
		RTC->RIIR.AI &= RTC_INT_ALARM; /* Clear Interrupt Pending Bit */
	}
	if(RTC->RIIR.TI & (RTC_INT_COUNTER_INCREASE))
	{
		REG32WRITE(RTC->RIIR,TI,1);		        /* Clear Interrupt Pending Bit */
	}

}

/***************************************************************************//**
 * @brief     Initializes the RTC peripheral
 * @param[in] None
 * @return    None
*******************************************************************************/
COX_Status NUC1xx_RTC_Init (void)
{
	int32_t i32i =0;
	int32_t i32delay=1000;
	/*-----------------------------------------------------------------------------------------------------*/
	/* Initial time data struct and some parameters.                                                       */
	/*-----------------------------------------------------------------------------------------------------*/

	UNLOCKREG();
	/* Enable 32K Clock */
	REG32WRITE(SYSCLK->PWRCON,XTL32K_EN,1);

	/* Waiting for 32K stable */
	while(i32delay--);

	/* Enable RTC Clock */
	REG32WRITE(SYSCLK->APBCLK,RTC_EN,1);
	LOCKREG();

	/*-----------------------------------------------------------------------------------------------------*/
	/* When RTC is power on, write 0xa5eb1357 to RTC_INIR to reset all logic.                              */
	/*-----------------------------------------------------------------------------------------------------*/
	RTC->INIR = DRVRTC_INIT_KEY;

	for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
	{

		if(RTC->INIR == 0x1)  /* Check RTC_INIR[0] to find out RTC reset signal */
		{
			break;
    }
	}

	if (i32i == DRVRTC_WAIT_COUNT)
	{
		return COX_ERROR;
	}
	NVIC_EnableIRQ(RTC_IRQn);

	return COX_SUCCESS;
}


/***************************************************************************//**
 * @brief     Start RTC Timer
 * @param[in] None
 * @return    None
*******************************************************************************/
void NUC1xx_RTC_Start (void)
{
	RTC->INIR |=  ((uint32_t)1<<0);
}

/***************************************************************************//**
 * @brief     Stop RTC Timer
 * @param[in] None
 * @return    None
*******************************************************************************/
void NUC1xx_RTC_Stop (void)
{
	RTC->INIR &= ~((uint32_t)1<<0);
}

/***************************************************************************//**
 * @brief     Read RTC Timer
 * @param[in] tm : Pointer to a Time structure that contains time value
 * @return    None
*******************************************************************************/
void NUC1xx_RTC_Read (Time *tm)
{
	uint32_t time_and_date;
	g_u32hiYear  = RTC->CLR.YEAR10 & RTC_YEAR_HI_MASK ;
	g_u32loYear  = RTC->CLR.YEAR1  & RTC_YEAR_LO_MASK;
	g_u32hiMonth = RTC->CLR.MON10  & RTC_MONTH_HI_MASK;
	g_u32loMonth = RTC->CLR.MON1   & RTC_MONTH_LO_MASK;
	g_u32hiDay   = RTC->CLR.DAY10  & RTC_DAY_HI_MASK;
	g_u32loDay   = RTC->CLR.DAY1   & RTC_DAY_LO_MASK;

	g_u32hiHour =  RTC->TLR.HR10  &  RTC_HOUR_HI_MASK;
	g_u32loHour =  RTC->TLR.HR1   &  RTC_HOUR_LO_MASK;
	g_u32hiMin  =  RTC->TLR.MIN10 &  RTC_MIN_HI_MASK;
	g_u32loMin  =  RTC->TLR.MIN1  &  RTC_MIN_LO_MASK;
	g_u32hiSec  =  RTC->TLR.SEC10 &  RTC_SEC_HI_MASK;
	g_u32loSec  =  RTC->TLR.SEC1  &  RTC_SEC_LO_MASK;


	/* Compute the year, month, day  */
	time_and_date = 	  (g_u32hiYear * 10);									/* Compute to 20XX year */
	time_and_date +=   g_u32loYear;
	tm->year   =   time_and_date  + 2000;

	time_and_date = (g_u32hiMonth * 10);									    /* Compute 0~12 month */
	tm->mon = time_and_date + g_u32loMonth;

	time_and_date = (g_u32hiDay * 10);										/* Compute 0~31 day */
	tm->mday   =  time_and_date  + g_u32loDay;

	/* comoute the time hour,minute,second */
	time_and_date  = (g_u32hiHour * 10);                     	/* RTC_CLOCK_24 */
	time_and_date += g_u32loHour;
	tm ->hour   = time_and_date;

	time_and_date = (g_u32hiMin  * 10);
	time_and_date+= g_u32loMin;
	tm->min = time_and_date;

	time_and_date = (g_u32hiSec  * 10);
	time_and_date+= g_u32loSec;
	tm->sec = time_and_date;

}

/***************************************************************************//**
 * @brief     Set RTC Time
 * @param[in] tm : Pointer to a Time structure that contains time value
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR : Failed to set
*******************************************************************************/
COX_Status NUC1xx_RTC_Write (Time *tm)
{
	uint32_t time_and_date;
	int32_t i32i = 0;

	int i32retry = 100;
	/*-----------------------------------------------------------------------------------------------------*/
	/* Important, call RTC_Open() before write data into any register.                                     */
	/*-----------------------------------------------------------------------------------------------------*/

	/*-------------------------------------------------------------------------------------------------*/
	/* After 200ms, Access enable wiil auto-clear. As soon as possible to do your setting              */
	/*-------------------------------------------------------------------------------------------------*/
	do
	{
		i32i = 0;
		REG32WRITE(RTC->AER,AER,0xA965);

		for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
		{
			/*-------------------------------------------------------------------------------------------------*/
    	/* check RTC_AER[16] to find out RTC write enable                                                  */
			/*-------------------------------------------------------------------------------------------------*/
			REG32WRITE(RTC->AER,AER,0xA965);
  		if (RTC->AER.ENF ==1)
				break;

		}
		if (i32i == DRVRTC_WAIT_COUNT)
		{

			i32retry--;
			if(!i32retry)
			return COX_ERROR;
		}

	}while(i32i == DRVRTC_WAIT_COUNT);
	REG32WRITE(RTC->TSSR,HR24,1);
	REG32WRITE(RTC->DWR,DWR,tm->wday);
	/*---------------------------------------------------------------------------------------------*/
	/* Second, set RTC time data.                                                                  */
	/*---------------------------------------------------------------------------------------------*/
	time_and_date     = ((tm->year - 2000) / 10) << 20;
	time_and_date    |= (((tm->year - 2000) % 10) << 16);
	time_and_date    |= ((tm->mon  / 10) << 12);
	time_and_date    |= ((tm->mon  % 10) << 8);
	time_and_date    |= ((tm->mday    / 10) << 4);
	time_and_date    |= (tm->mday     % 10);
	/*-------------------------------------------------------------------------------------------------*/
	/* After 200ms, Access enable wiil auto-clear. As soon as possible to do your setting              */
	/*-------------------------------------------------------------------------------------------------*/
	i32retry = 100;
	do
	{
		i32i = 0;
		REG32WRITE(RTC->AER,AER,0xA965);

		for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
		{
			/*-------------------------------------------------------------------------------------------------*/
			/* check RTC_AER[16] to find out RTC write enable                                                  */
			/*-------------------------------------------------------------------------------------------------*/
			REG32WRITE(RTC->AER,AER,0xA965);

			if (RTC->AER.ENF ==1)
				break;
		}
		if (i32i == DRVRTC_WAIT_COUNT)
		{

			i32retry--;
  		if(!i32retry)
			return COX_ERROR;
		}
	}while(i32i == DRVRTC_WAIT_COUNT);
	outpw(&RTC->CLR, (uint32_t)time_and_date);

	time_and_date     = ((tm->hour / 10) << 20);
	time_and_date    |= ((tm->hour % 10) << 16);
	time_and_date    |= ((tm->min / 10) << 12);
	time_and_date    |= ((tm->min % 10) << 8);
	time_and_date    |= ((tm->sec / 10) << 4);
	time_and_date    |= (tm->sec % 10);

	/*-------------------------------------------------------------------------------------------------*/
	/* After 200ms, Access enable wiil auto-clear. As soon as possible to do your setting              */
	/*-------------------------------------------------------------------------------------------------*/
	i32retry = 100;
	do
	{
		i32i = 0;

			REG32WRITE(RTC->AER,AER,0xA965);

		for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
		{
			/*-------------------------------------------------------------------------------------------------*/
    	/* check RTC_AER[16] to find out RTC write enable                                                  */
			/*-------------------------------------------------------------------------------------------------*/
			REG32WRITE(RTC->AER,AER,0xA965);

			if (RTC->AER.ENF ==1)
				break;
		}
		if (i32i == DRVRTC_WAIT_COUNT)
		{
			i32retry--;
  		if(!i32retry)
			return COX_ERROR;
		}
	}while(i32i == DRVRTC_WAIT_COUNT);
	outpw(&RTC->TLR, (uint32_t)time_and_date);
	return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Set Alarm time
 * @param[in] tm : Pointer to a Time structure that contains alarm time value
 * @param[in] handler : Alarm handler function
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR : Failed to set
*******************************************************************************/
COX_Status NUC1xx_RTC_Alarm (Time *tm, RTC_Event_Handler handler)
{
	int32_t i32i = 0;
	int i32retry = 100;
	uint32_t arm_timeanddate;
	if(handler != COX_NULL)
	{
		if(tm == COX_NULL) return COX_ERROR;
		REG32WRITE(RTC->TSSR,HR24,1);
		REG32WRITE(RTC->DWR,DWR,tm->wday);
		/*---------------------------------------------------------------------------------------------*/
		/* Second, set RTC time data.                                                                  */
		/*---------------------------------------------------------------------------------------------*/
		arm_timeanddate = ((tm->year - 2000) / 10) << 20;
		arm_timeanddate |= (((tm->year - 2000) % 10) << 16);
		arm_timeanddate |= ((tm->mon / 10) << 12);
		arm_timeanddate |= ((tm->mon % 10) << 8);
		arm_timeanddate |= ((tm->mday / 10) << 4);
		arm_timeanddate |= (tm->mday % 10);
		/*-------------------------------------------------------------------------------------------------*/
		/* After 200ms, Access enable wiil auto-clear. As soon as possible to do your setting              */
		/*-------------------------------------------------------------------------------------------------*/
		do
		{
			i32i = 0;

			REG32WRITE(RTC->AER,AER,0xA965);

    	for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
	  	{
    		/*-------------------------------------------------------------------------------------------------*/
    		/* check RTC_AER[16] to find out RTC write enable                                                  */
    		/*-------------------------------------------------------------------------------------------------*/
			REG32WRITE(RTC->AER,AER,0xA965);

	    	if (RTC->AER.ENF ==1)
		  	break;
  		}
	  	if (i32i == DRVRTC_WAIT_COUNT)
    	{
		  	i32retry--;
      	if(!i32retry)
	   		return COX_ERROR;
    	}
		}while(i32i == DRVRTC_WAIT_COUNT);
    outpw(&RTC->CAR, arm_timeanddate);
    arm_timeanddate     = ((tm->hour / 10) << 20);
    arm_timeanddate    |= ((tm->hour % 10) << 16);
    arm_timeanddate    |= ((tm->min / 10) << 12);
    arm_timeanddate    |= ((tm->min % 10) << 8);
    arm_timeanddate    |= ((tm->sec / 10) << 4);
    arm_timeanddate    |= (tm->sec % 10);
    /*-------------------------------------------------------------------------------------------------*/
    /* After 200ms, Access enable wiil auto-clear. As soon as possible to do your setting              */
    /*-------------------------------------------------------------------------------------------------*/
    i32retry = 100;
		do
		{
	  	i32i = 0;

			REG32WRITE(RTC->AER,AER,0xA965);

    	for (i32i = 0 ; i32i < DRVRTC_WAIT_COUNT ; i32i++)
	  	{
    		/*-------------------------------------------------------------------------------------------------*/
      	/* check RTC_AER[16] to find out RTC write enable                                                  */
      	/*-------------------------------------------------------------------------------------------------*/
			REG32WRITE(RTC->AER,AER,0xA965);

	    	if (RTC->AER.ENF ==1)
		  		break;
  		}
	  	if (i32i == DRVRTC_WAIT_COUNT)
    	{
				i32retry--;
      	if(!i32retry)
	   		return COX_ERROR;
    	}
		}while(i32i == DRVRTC_WAIT_COUNT);
		outpw(&RTC->TAR, arm_timeanddate);
		REG32WRITE(RTC->RIER,AIER,1);/* Setup Alarm event handler */
		REG32WRITE(RTC->RIIR,AI,1); /* Enable Alarm Event */
	}
	else
	{
		REG32WRITE(RTC->RIER,AIER,0);
		REG32WRITE(RTC->RIIR,AI,0);            /* Disable Alarm event handler */
	}
	return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Set RTC Event
 * @param[in] event : Event type, should be one of the following :
 *                     - RTC_EVENT_INC_NONE : None event, disable all event
 *                     - RTC_EVENT_INC_SEC  : Second Increment Event
 *                     - RTC_EVENT_INC_MIN  : Minute Increment Event
 * @param[in] handler : Event handler function
 * @param[in] art     : Argument, not used
 * @return    - COX_ERR_OK : Success
 *            - COX_ERR_FAIL : Failed to set
 * Note : Only one event is available at the same time.
*******************************************************************************/
COX_Status NUC1xx_RTC_Event (uint8_t event, RTC_Event_Handler handler, uint32_t arg)
{
	COX_Status err = COX_ERROR;
	uint32_t getinttype = 0;

	switch(event) {
		case RTC_EVENT_INC_NONE :  /* None Interrupt            */
			rtc_inc_event = COX_NULL;
			rtc_alarm_event = COX_NULL;
			REG32WRITE(RTC->RIER,AIER,0);
			REG32WRITE(RTC->RIER,TIER,0);
			err =  COX_SUCCESS;
			break;
		case RTC_EVENT_INT :   /* INT Increment Interrupt */
    	getinttype = RTC_AIER_INT;   break;

		case RTC_EVENT_ALARM :    /* ALARM Increment Interrupt */
     	getinttype = RTC_AIER_ALARM;   break;
	}
	if((getinttype != 0) && (handler != COX_NULL))
	{
		if(getinttype == RTC_AIER_INT)
		{
			REG32WRITE(RTC->RIER,TIER,1);
			REG32WRITE(RTC->RIIR,TI,1);
			rtc_inc_event = handler;
			err = COX_SUCCESS;
		}
		else
		{
			REG32WRITE(RTC->RIER,AIER,1);
			REG32WRITE(RTC->RIIR,AI,1);
			rtc_alarm_event = handler;
			err = COX_SUCCESS;
	 	}
	}
	return err;
}

/***************************************************************************//**
 * @brief Define RTC Interface
*******************************************************************************/
COX_RTC_PI pi_rtc = {
  NUC1xx_RTC_Init,
  NUC1xx_RTC_Start,
  NUC1xx_RTC_Stop,
  NUC1xx_RTC_Read,
  NUC1xx_RTC_Write,
  NUC1xx_RTC_Alarm,
  NUC1xx_RTC_Event,
};

