/***************************************************************************//**
 * @file      : nuc_pwm.c
 * @brief     : NUC1xx PWM Peripheral Interface
 * @version   : V1.0
 * @date      : 8. STP. 2010
 * @author    : CooCox 
*******************************************************************************/
#include "nuc_pwm.h"
#include "nuc1xx.h"

/***************************************************************************//**
 *  PWM Clcok Selector						                                                               
*******************************************************************************/
#define DRVPWM_CLOCK_DIV_1     4
#define DRVPWM_CLOCK_DIV_2     0
#define DRVPWM_CLOCK_DIV_4     1
#define DRVPWM_CLOCK_DIV_8     2
#define DRVPWM_CLOCK_DIV_16    3

/***************************************************************************//**
 *  PWM Timer 								                                                               
*******************************************************************************/
#define	DRVPWM_TIMER0          0x00
#define	DRVPWM_TIMER1          0x01
#define	DRVPWM_TIMER2          0x02
#define	DRVPWM_TIMER3          0x03
#define	DRVPWM_TIMER4          0x04
#define	DRVPWM_TIMER5          0x05
#define	DRVPWM_TIMER6          0x06
#define	DRVPWM_TIMER7          0x07

/**************************************************************************//**
 *  PWM Clcok Source Selector						                                                       
******************************************************************************/
#define DRVPWM_EXT_12M		   0
#define DRVPWM_EXT_32K		   1
#define DRVPWM_HCLK     	   2
#define DRVPWM_INTERNAL_22M    3


/***************************************************************************//**
 *  PWM Mode Selector						                                                              
*******************************************************************************/
#define DRVPWM_TOGGLE_MODE		1
#define DRVPWM_ONE_SHOT_MODE	0

#define NUC_PWM_CHANNEL_INVALID       0xFF

/***************************************************************************//**
 * @brief     Setup PWM Output Pin
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @return 
 *         - COX_SUCCESS : success
 *         - COX_ERROR : The PWM Output Pin was not supported
*******************************************************************************/
static COX_Status NUC_PWM_PinCfg (uint16_t pwmout)
{


	switch(pwmout) {
	case NUC_PWM_CHANNEL0 :
		REG32WRITE(SYS->GPAMFP, PWM0, 1); REG32WRITE(PWMA->POE, PWM0, 1);break;   /* PA.12 */
	case NUC_PWM_CHANNEL1 :
		REG32WRITE(SYS->GPAMFP, PWM1, 1); REG32WRITE(PWMA->POE, PWM1, 1);break;   /* PA.13 */
	case NUC_PWM_CHANNEL2 :
		REG32WRITE(SYS->GPAMFP, PWM2, 1); REG32WRITE(PWMA->POE, PWM2, 1);break;   /* PA.14 */
	case NUC_PWM_CHANNEL3 : 
		REG32WRITE(SYS->GPAMFP, PWM3_I2SMCLK, 1);REG32WRITE(SYS->ALTMFP, PA15_I2SMCLK, 0); REG32WRITE(PWMA->POE, PWM3, 1);break;   /* PA.15 */
	case NUC_PWM_CHANNEL4 : 
		REG32WRITE(SYS->ALTMFP, PB11_PWM4, 1);REG32WRITE(SYS->GPBMFP, TM3_PWM4, 1); REG32WRITE(PWMB->POE, PWM0, 1);break;          /* PB.11 */
	case NUC_PWM_CHANNEL5 :
		REG32WRITE(SYS->GPEMFP, PWM5, 1); REG32WRITE(PWMB->POE, PWM1, 1);break;   /* PE.5   */
	case NUC_PWM_CHANNEL6 :
		REG32WRITE(SYS->GPEMFP, PWM6, 1); REG32WRITE(PWMB->POE, PWM2, 1);break;   /* PE.0   */
	case NUC_PWM_CHANNEL7 :
		REG32WRITE(SYS->GPEMFP, PWM7, 1); REG32WRITE(PWMB->POE, PWM3, 1);break;   /* PE.1   */
	default : return COX_ERROR;
	}



  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Get PWM channel number via PWM Output Pin
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @return    PWM channel number
*******************************************************************************/
uint8_t NUC_PWM_Channel (uint16_t pwmout)
{  
	uint8_t chn;

	switch(pwmout) 
	{
		case NUC_PWM_CHANNEL0 : chn = 0x0; break;   /* PA.12 */
		case NUC_PWM_CHANNEL1 : chn = 0x1; break;   /* PA.13 */  
		case NUC_PWM_CHANNEL2 : chn = 0x2; break;   /* PA.14 */
		case NUC_PWM_CHANNEL3 : chn = 0x3; break;   /* PA.15 */
		case NUC_PWM_CHANNEL4 : chn = 0x4; break;   /* PB.11 */  
		case NUC_PWM_CHANNEL5 : chn = 0x5; break;   /* PE.5   */
		case NUC_PWM_CHANNEL6 : chn = 0x6; break;   /* PE.0   */
		case NUC_PWM_CHANNEL7 : chn = 0x7; break;   /* PE.1   */  
		default : return 0xFF;
	}

	return chn;     
}

/***************************************************************************//**
 * @brief     Initializes PWM Peripheral
 * @param[in] None
 * @return    Always SUCCESS
*******************************************************************************/
COX_Status NUC_PWM_Init (void)
{  

	REG32WRITE(SYSCLK->APBCLK, PWM01_EN, 1);
	REG32WRITE(SYSCLK->APBCLK, PWM23_EN, 1);
	REG32WRITE(SYSCLK->APBCLK, PWM45_EN, 1);
	REG32WRITE(SYSCLK->APBCLK, PWM67_EN, 1);
	
	REG32WRITE(SYS->IPRSTC2, PWM03_RST, 1);
	REG32WRITE(SYS->IPRSTC2, PWM03_RST, 0);
	REG32WRITE(SYS->IPRSTC2, PWM47_RST, 1);
	REG32WRITE(SYS->IPRSTC2, PWM47_RST, 0);
	
	REG32WRITE(SYSCLK->CLKSEL1, PWM01_S, DRVPWM_EXT_12M);
	REG32WRITE(SYSCLK->CLKSEL1, PWM23_S, DRVPWM_EXT_12M);
	REG32WRITE(SYSCLK->CLKSEL2, PWM45_S, DRVPWM_EXT_12M);
	REG32WRITE(SYSCLK->CLKSEL2, PWM67_S, DRVPWM_EXT_12M);
	
	REG32WRITE(PWMA->PCR, CH0MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMA->PCR, CH1MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMA->PCR, CH2MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMA->PCR, CH3MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMB->PCR, CH0MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMB->PCR, CH1MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMB->PCR, CH2MOD, DRVPWM_TOGGLE_MODE);
	REG32WRITE(PWMB->PCR, CH3MOD, DRVPWM_TOGGLE_MODE);
	  
	return COX_SUCCESS;     
}

/***************************************************************************//**
 * @brief     Setup PWM Output Period in second
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @param[in] period : PWM Output Period in second
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output or With value were not supported
 *
 * @NOTE : The NUC1xx PWM hardware have the different  period value between 
 *         all outputs. Therefore, The period value can be set independently for each 
 *         output. And the pulsewidth can be set independently for each 
 *         output too.For use ,you must set both of them.So each PWM are independent.
*******************************************************************************/
COX_Status NUC_PWM_Period (uint16_t pwmout, float period)
{

	uint32_t	u32Freqout;
	uint32_t 	u32Frequency;
	uint8_t	 u8Divider;
	uint16_t u16PreScale;
	uint16_t	u16Duty,u16cnrData;
	uint8_t chn;
	chn = NUC_PWM_Channel (pwmout);
	u32Freqout = 12000000;
	
	u32Frequency =  u32Freqout *period;		
	
	if (u32Frequency > 0x10000000)
		return COX_ERROR;
		
	u8Divider = 1;			
		
	if (u32Frequency < 0x20000)
		u16PreScale = 2;	
	else
	{
		u16PreScale = u32Frequency / 65536;	
						
		if (u32Frequency / u16PreScale > 65536)
			u16PreScale++;
		
		if (u16PreScale > 256)
		{
			uint8_t u8i = 0;
			
			u16PreScale = 256;
			u32Frequency = u32Frequency / u16PreScale;
			
			u8Divider = u32Frequency / 65536;				
			
			if(u32Frequency / u8Divider > 65536)
				u8Divider++;				
			
			while(1)	
			{
				if((1 << u8i++) > u8Divider)
					break;
			}
			
			u8Divider = 1 << (u8i - 1);
			
			if (u8Divider > 16)
				return 0;	
				
			u32Frequency = u32Frequency * u16PreScale;						
		}		
				
	}
	u16Duty = (uint16_t )(u32Frequency/u16PreScale/u8Divider);
	
	u32Frequency = (u32Freqout / u16PreScale / u8Divider) / u16Duty;	
			
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER2:
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->PPR, CP23, u16PreScale - 1);
			break;
		case DRVPWM_TIMER4:
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER6:
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->PPR, CP23, u16PreScale - 1);
			break;
	}
	
	switch(u8Divider)
	{
		case 1:
			u8Divider = DRVPWM_CLOCK_DIV_1;
			break;
		case 2:
			u8Divider = DRVPWM_CLOCK_DIV_2;			
			break;			
		case 4:
			u8Divider = DRVPWM_CLOCK_DIV_4;			
			break;			
		case 8:
			u8Divider = DRVPWM_CLOCK_DIV_8;			
			break;			
		case 16:
			u8Divider = DRVPWM_CLOCK_DIV_16;			
			break;		
	}				
				

	 	u16cnrData = u16Duty - 1;
	
	
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
			REG32WRITE(PWMA->CSR, CSR0, u8Divider);
			PWMA->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->CSR, CSR1, u8Divider);
			PWMA->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER2:
			REG32WRITE(PWMA->CSR, CSR2, u8Divider);
			PWMA->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->CSR, CSR3, u8Divider);
			PWMA->CNR3 = u16cnrData;
			break;
		case DRVPWM_TIMER4:
			REG32WRITE(PWMB->CSR, CSR0, u8Divider);
			PWMB->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->CSR, CSR1, u8Divider);
			PWMB->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER6:
			REG32WRITE(PWMB->CSR, CSR2, u8Divider);
			PWMB->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->CSR, CSR3, u8Divider);
			PWMB->CNR3 = u16cnrData;
			break;
	}
 
  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Set the PWM Pulsewidth in second
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @param[in] period : PWM Pulsewidth in second
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output or With value were not supported
*******************************************************************************/
COX_Status NUC_PWM_Width (uint16_t pwmout, float width)
{
	uint32_t	u32Freqout;
	uint32_t 	u32Frequency;
	uint8_t	 u8Divider;
	uint16_t u16PreScale;
	uint16_t	u16Duty,u16cnrData;
	uint8_t chn;
	NUC_PWM_PinCfg(pwmout);
	chn = NUC_PWM_Channel (pwmout);
	u32Freqout = 12000000;
	
	u32Frequency =  u32Freqout *width;		
	
	if (u32Frequency > 0x10000000)
		return COX_ERROR;
		
	u8Divider = 1;			
		
	if (u32Frequency < 0x20000)
		u16PreScale = 2;	
	else
	{
		u16PreScale = u32Frequency / 65536;	
						
		if (u32Frequency / u16PreScale > 65536)
			u16PreScale++;
		
		if (u16PreScale > 256)
		{
			uint8_t u8i = 0;
			
			u16PreScale = 256;
			u32Frequency = u32Frequency / u16PreScale;
			
			u8Divider = u32Frequency / 65536;				
			
			if(u32Frequency / u8Divider > 65536)
				u8Divider++;				
			
			while(1)	
			{
				if((1 << u8i++) > u8Divider)
					break;
			}
			
			u8Divider = 1 << (u8i - 1);
			
			if (u8Divider > 16)
				return 0;	
				
			u32Frequency = u32Frequency * u16PreScale;						
		}		
				
	}
	u16Duty = (uint16_t )(u32Frequency/u16PreScale/u8Divider);
	
	u32Frequency = (u32Freqout / u16PreScale / u8Divider) / u16Duty;	
			
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER2:
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->PPR, CP23, u16PreScale - 1);
			break;
		case DRVPWM_TIMER4:
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER6:
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->PPR, CP23, u16PreScale - 1);
			break;
	}
	
	switch(u8Divider)
	{
		case 1:
			u8Divider = DRVPWM_CLOCK_DIV_1;
			break;
		case 2:
			u8Divider = DRVPWM_CLOCK_DIV_2;			
			break;			
		case 4:
			u8Divider = DRVPWM_CLOCK_DIV_4;			
			break;			
		case 8:
			u8Divider = DRVPWM_CLOCK_DIV_8;			
			break;			
		case 16:
			u8Divider = DRVPWM_CLOCK_DIV_16;			
			break;		
	}				
				

	 	u16cnrData = u16Duty - 1;
	
	
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
			REG32WRITE(PWMA->CSR, CSR0, u8Divider);
			PWMA->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->CSR, CSR1, u8Divider);
			PWMA->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER2:
			REG32WRITE(PWMA->CSR, CSR2, u8Divider);
			PWMA->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->CSR, CSR3, u8Divider);
			PWMA->CNR3 = u16cnrData;
			break;
		case DRVPWM_TIMER4:
			REG32WRITE(PWMB->CSR, CSR0, u8Divider);
			PWMB->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->CSR, CSR1, u8Divider);
			PWMB->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER6:
			REG32WRITE(PWMB->CSR, CSR2, u8Divider);
			PWMB->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->CSR, CSR3, u8Divider);
			PWMB->CNR3 = u16cnrData;
			break;
	}
 
  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Setup PWM Output Period in microsecond
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @param[in] period : PWM Output Period in microsecond
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output was not supported
 *
 * @NOTE : The LPC17xx PWM hardware is limited to share the period value between 
 *         all outputs. Therefore, if you change the period of one output, you 
 *         change them all. The pulsewidth can be set independently for each 
 *         output.
*******************************************************************************/
COX_Status NUC_PWM_usPeriod (uint16_t pwmout, uint32_t period)
{
 
	uint32_t	u32Freqout;
	uint32_t 	u32Frequency;
	uint8_t	 u8Divider;
	uint16_t u16PreScale;
	uint16_t	u16Duty,u16cnrData;
	uint8_t chn;
	chn = NUC_PWM_Channel (pwmout);
	u32Freqout = 12000000;
	
	u32Frequency =  u32Freqout/1000 *period/1000;		
	
	if (u32Frequency > 0x10000000)
		return COX_ERROR;
		
	u8Divider = 1;			
		
	if (u32Frequency < 0x20000)
		u16PreScale = 2;	
	else
	{
		u16PreScale = u32Frequency / 65536;	
						
		if (u32Frequency / u16PreScale > 65536)
			u16PreScale++;
		
		if (u16PreScale > 256)
		{
			uint8_t u8i = 0;
			
			u16PreScale = 256;
			u32Frequency = u32Frequency / u16PreScale;
			
			u8Divider = u32Frequency / 65536;				
			
			if(u32Frequency / u8Divider > 65536)
				u8Divider++;				
			
			while(1)	
			{
				if((1 << u8i++) > u8Divider)
					break;
			}
			
			u8Divider = 1 << (u8i - 1);
			
			if (u8Divider > 16)
				return 0;	
				
			u32Frequency = u32Frequency * u16PreScale;						
		}		
				
	}
	u16Duty = (uint16_t )(u32Frequency/u16PreScale/u8Divider);
	
	u32Frequency = (u32Freqout / u16PreScale / u8Divider) / u16Duty;	
			
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER2:
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->PPR, CP23, u16PreScale - 1);
			break;
		case DRVPWM_TIMER4:
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER6:
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->PPR, CP23, u16PreScale - 1);
			break;
	}
	
	switch(u8Divider)
	{
		case 1:
			u8Divider = DRVPWM_CLOCK_DIV_1;
			break;
		case 2:
			u8Divider = DRVPWM_CLOCK_DIV_2;			
			break;			
		case 4:
			u8Divider = DRVPWM_CLOCK_DIV_4;			
			break;			
		case 8:
			u8Divider = DRVPWM_CLOCK_DIV_8;			
			break;			
		case 16:
			u8Divider = DRVPWM_CLOCK_DIV_16;			
			break;		
	}				
				

	 	u16cnrData = u16Duty - 1;
	
	
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
			REG32WRITE(PWMA->CSR, CSR0, u8Divider);
			PWMA->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->CSR, CSR1, u8Divider);
			PWMA->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER2:
			REG32WRITE(PWMA->CSR, CSR2, u8Divider);
			PWMA->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->CSR, CSR3, u8Divider);
			PWMA->CNR3 = u16cnrData;
			break;
		case DRVPWM_TIMER4:
			REG32WRITE(PWMB->CSR, CSR0, u8Divider);
			PWMB->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->CSR, CSR1, u8Divider);
			PWMB->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER6:
			REG32WRITE(PWMB->CSR, CSR2, u8Divider);
			PWMB->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->CSR, CSR3, u8Divider);
			PWMB->CNR3 = u16cnrData;
			break;
	}
 
  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Set the PWM Pulsewidth in microsecond
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @param[in] period : PWM Pulsewidth in microsecond
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output or With value were not supported
*******************************************************************************/
COX_Status NUC_PWM_usWidth (uint16_t pwmout, uint32_t width)
{
	uint32_t	u32Freqout;
	uint32_t 	u32Frequency;
	uint8_t	 u8Divider;
	uint16_t u16PreScale;
	uint16_t	u16Duty,u16cnrData;
	uint8_t chn;
	NUC_PWM_PinCfg(pwmout);
	chn = NUC_PWM_Channel (pwmout);
	u32Freqout = 12000000;
	
	u32Frequency =  u32Freqout/1000 *width/1000;		
	
	if (u32Frequency > 0x10000000)
		return COX_ERROR;
		
	u8Divider = 1;			
		
	if (u32Frequency < 0x20000)
		u16PreScale = 2;	
	else
	{
		u16PreScale = u32Frequency / 65536;	
						
		if (u32Frequency / u16PreScale > 65536)
			u16PreScale++;
		
		if (u16PreScale > 256)
		{
			uint8_t u8i = 0;
			
			u16PreScale = 256;
			u32Frequency = u32Frequency / u16PreScale;
			
			u8Divider = u32Frequency / 65536;				
			
			if(u32Frequency / u8Divider > 65536)
				u8Divider++;				
			
			while(1)	
			{
				if((1 << u8i++) > u8Divider)
					break;
			}
			
			u8Divider = 1 << (u8i - 1);
			
			if (u8Divider > 16)
				return 0;	
				
			u32Frequency = u32Frequency * u16PreScale;						
		}		
				
	}
	u16Duty = (uint16_t )(u32Frequency/u16PreScale/u8Divider);
	
	u32Frequency = (u32Freqout / u16PreScale / u8Divider) / u16Duty;	
			
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER2:
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->PPR, CP23, u16PreScale - 1);
			break;
		case DRVPWM_TIMER4:
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->PPR, CP01, u16PreScale - 1);
			break;
		case DRVPWM_TIMER6:
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->PPR,CP23, u16PreScale - 1);
			break;
	}
	
	switch(u8Divider)
	{
		case 1:
			u8Divider = DRVPWM_CLOCK_DIV_1;
			break;
		case 2:
			u8Divider = DRVPWM_CLOCK_DIV_2;			
			break;			
		case 4:
			u8Divider = DRVPWM_CLOCK_DIV_4;			
			break;			
		case 8:
			u8Divider = DRVPWM_CLOCK_DIV_8;			
			break;			
		case 16:
			u8Divider = DRVPWM_CLOCK_DIV_16;
			break;		
	}				
				

	 	u16cnrData = u16Duty - 1;
	
	
	switch (chn & 0x07)
	{
		case DRVPWM_TIMER0:
			REG32WRITE(PWMA->CSR, CSR0, u8Divider);
			PWMA->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER1:
			REG32WRITE(PWMA->CSR, CSR1, u8Divider);
			PWMA->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER2:
			REG32WRITE(PWMA->CSR, CSR2, u8Divider);
			PWMA->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER3:
			REG32WRITE(PWMA->CSR, CSR3, u8Divider);
			PWMA->CNR3 = u16cnrData;
			break;
		case DRVPWM_TIMER4:
			REG32WRITE(PWMB->CSR, CSR0, u8Divider);
			PWMB->CNR0 = u16cnrData;
			break;
		case DRVPWM_TIMER5:
			REG32WRITE(PWMB->CSR, CSR1, u8Divider);
			PWMB->CNR1 = u16cnrData;
			break;
		case DRVPWM_TIMER6:
			REG32WRITE(PWMB->CSR, CSR2, u8Divider);
			PWMB->CNR2 = u16cnrData;
			break;
		case DRVPWM_TIMER7:
			REG32WRITE(PWMB->CSR, CSR3, u8Divider);
			PWMB->CNR3 = u16cnrData;
			break;
	}
 
  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Set the ouput duty-cycle, specified as a percentage
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @param[in] duty : duty-cycle, specified as a percentage
 * @return    - COX_SUCCESS : success
 *            - COX_ERROR   : The Duty-cycle was not supported
*******************************************************************************/
COX_Status NUC_PWM_Duty (uint16_t pwmout, float duty)
{
	uint32_t value;
	uint8_t  chn;

	if(duty > 1) return COX_ERROR;  

	NUC_PWM_PinCfg(pwmout);

	chn = NUC_PWM_Channel (pwmout);
	if(chn == NUC_PWM_CHANNEL_INVALID) {
	return COX_ERROR;
	}

	switch(chn) {
	case 0: value = PWMA->CNR0;value *= duty;PWMA->CMR0 = value; break;
	case 1: value = PWMA->CNR1;value *= duty;PWMA->CMR1 = value; break;
	case 2: value = PWMA->CNR2;value *= duty;PWMA->CMR2 = value; break;
	case 3: value = PWMA->CNR3;value *= duty;PWMA->CMR3 = value; break;
	case 4: value = PWMB->CNR0;value *= duty;PWMB->CMR0 = value; break;
	case 5: value = PWMB->CNR1;value *= duty;PWMB->CMR1 = value; break;
	case 6: value = PWMB->CNR2;value *= duty;PWMB->CMR2 = value; break;
	case 7: value = PWMB->CNR3;value *= duty;PWMB->CMR3 = value; break;
	};  



	return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Initiate PWM Output
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output was not supported
*******************************************************************************/
COX_Status NUC_PWM_Start (uint16_t pwmout)
{
	uint8_t chn;
  	chn = NUC_PWM_Channel (pwmout);
	switch (chn & 0x07)
	{
		case 0:
			REG32WRITE(PWMA->PCR, CH0EN, 1);
			break;
		case 1:
			REG32WRITE(PWMA->PCR, CH1EN, 1);
			break;
		case 2:
			REG32WRITE(PWMA->PCR, CH2EN, 1);
			break;
		case 3:
			REG32WRITE(PWMA->PCR, CH3EN, 1);
			break;
		case 4:
			REG32WRITE(PWMB->PCR, CH0EN, 1);
			break;
		case 5:
			REG32WRITE(PWMB->PCR, CH1EN, 1);
			break;
		case 6:
			REG32WRITE(PWMB->PCR, CH2EN, 1);
			break;
		case 7:
			REG32WRITE(PWMB->PCR, CH3EN, 1);
			break;
	} 

  return COX_SUCCESS;  
}

/***************************************************************************//**
 * @brief     Stop PWM Output
 * @param[in] pwmout : PWM output Pin, should be one of the following
 *                     - NUC_PWM_CHANNEL0 : PWM Channel 0 Output, PA.12
 *                     - NUC_PWM_CHANNEL1 : PWM Channel 1 Output, PA.13
 *                     - NUC_PWM_CHANNEL2 : PWM Channel 2 Output, PA.14
 *                     - NUC_PWM_CHANNEL3 : PWM Channel 3 Output, PA.15
 *                     - NUC_PWM_CHANNEL4 : PWM Channel 4 Output, PB.11
 *                     - NUC_PWM_CHANNEL5 : PWM Channel 5 Output, PE.5
 *                     - NUC_PWM_CHANNEL6 : PWM Channel 6 Output, PE.0
 *                     - NUC_PWM_CHANNEL7 : PWM Channel 7 Output, PE.1
 * @return    - COX_SUCCESS : Success
 *            - COX_ERROR   : The PWM Output was not supported
*******************************************************************************/
COX_Status NUC_PWM_Stop (uint16_t pwmout)
{
	uint8_t chn;
  	chn = NUC_PWM_Channel (pwmout);
	switch (chn & 0x07)
	{
		case 0:
			REG32WRITE(PWMA->PCR, CH0EN, 0);
			break;
		case 1:
			REG32WRITE(PWMA->PCR, CH1EN, 0);
			break;
		case 2:
			REG32WRITE(PWMA->PCR, CH2EN, 0);
			break;
		case 3:
			REG32WRITE(PWMA->PCR, CH3EN, 0);
			break;
		case 4:
			REG32WRITE(PWMB->PCR, CH0EN, 0);
			break;
		case 5:
			REG32WRITE(PWMB->PCR, CH1EN, 0);
			break;
		case 6:
			REG32WRITE(PWMB->PCR, CH2EN, 0);
			break;
		case 7:
			REG32WRITE(PWMB->PCR, CH3EN, 0);
			break;
	} 

  return COX_SUCCESS;  
}

/***************************************************************************//**
 * Define NUC1xx COX Interface based on Timer
*******************************************************************************/
COX_PWM_PI pi_pwm = {
  NUC_PWM_Init,
  NUC_PWM_Period,
  NUC_PWM_Width,
  NUC_PWM_usPeriod,
  NUC_PWM_usWidth,
  NUC_PWM_Duty,
  NUC_PWM_Start,
  NUC_PWM_Stop
};
