/***************************************************************************//**
 * @file      : nuc_adc.c
 * @brief     : NUC1xx ADC Peripheral Interface 
 * @version   : V1.0
 * @date      : 17. August. 2010
 * @author    : CooCox
*******************************************************************************/           
#include "NUC_ADC.H"
#include "nuc1xx.h"


#define E_SYS_XTL12M   (12000000UL)

/***************************************************************************//**
 * Define global variables                                                           
*******************************************************************************/
uint8_t channel; 

                                                                                                                 
/***************************************************************************//**
 * @brief     Setup conversion clock 
 * @param[in] target_clock : Clock frequency in Hz
 * @return    Previous clock frequency in Hz
*******************************************************************************/
static uint32_t NUC_ADC_SetRate (uint32_t target_clock)
{
	uint32_t prescale, clk_bak, delay;

	if(target_clock == 0) return COX_ERROR;

	/* Unlock the locked registers before we can access them */
	UNLOCKREG(x);

	/* Enable the 12MHz oscillator oscillation */
	REG32WRITE(SYSCLK->PWRCON, XTL12M_EN, 1);

	/* Waiting for 12M Xtal stalble */
	for(delay=0;delay<10000;delay++)

	/* ADC clock source:12MHz oscillator oscillation */
	REG32WRITE(SYSCLK->CLKSEL1, ADC_S, 0);

	/* Get current clock */
	clk_bak = ( E_SYS_XTL12M / ((SYSCLK->CLKDIV.ADC_N) & 0xFF) + 1)/20;
	prescale = (E_SYS_XTL12M / target_clock / 20);


	/* ADC Peripheral Clock Selection */
	if(prescale > 0x100) prescale = 0x100;
	if(prescale != 0) prescale -= 1;

	SYSCLK->CLKDIV.ADC_N &= ~(0xFF);
	SYSCLK->CLKDIV.ADC_N |=  prescale;

	return clk_bak;
}

/***************************************************************************//**
 * @brief     Initialize the ADC peripheral
 * @param[in] clock : conversion clock 
 * @return    COX_SUCCESS or COX_ERROR
*******************************************************************************/
static COX_Status NUC_ADC_Init(uint32_t clock)
{ 
	/* reset ADC */
	REG32WRITE(SYS->IPRSTC2, ADC_RST, 1);
	REG32WRITE(SYS->IPRSTC2, ADC_RST, 0);
	/* Enable ADC power/clock */
	REG32WRITE(SYSCLK->APBCLK, ADC_EN, 1);
	/* Enable ADC */
	REG32WRITE(ADC->ADCR, ADEN, 1);

	NUC_ADC_SetRate(clock);

	return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Start a conversion and get the result
 * @param[in] channel : AD channel, should be :
 *                      -NUC_ADC_CHANNEL0 : AD0, PA.0 
 *                      -NUC_ADC_CHANNEL1 : AD1, PA.1 
 *                      -NUC_ADC_CHANNEL2 : AD2, PA.2
 *                      -NUC_ADC_CHANNEL3 : AD3, PA.3
 *                      -NUC_ADC_CHANNEL4 : AD4, PA.4
 *                      -NUC_ADC_CHANNEL5 : AD5, PA.5
 *                      -NUC_ADC_CHANNEL6 : AD6, PA.6
 *                      -NUC_ADC_CHANNEL7 : AD7, PA.7 
 * @return    Data conversion
*******************************************************************************/
static uint32_t NUC_ADC_Read(uint16_t ad_in)
{
	uint32_t adc_value;

	/* single end input */
	REG32WRITE(ADC->ADCR, DIFF, 0);
	/* single mode */
	REG32WRITE(ADC->ADCR, ADMD, 0);

	/* Select AD Function, configure PIO as input mode */
	if(ad_in == 0)
	{
		REG32WRITE(SYS->GPAMFP, ADC0, 1);    /* configure GPA0 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD0, 0);     /* configure GPA0 as input mode */
		channel = 0;
	}
	else if(ad_in == 1)
	{
		REG32WRITE(SYS->GPAMFP, ADC1, 1);   /* configure GPA1 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD1, 0);    /* configure GPA1 as input mode */
		channel = 1;
	}
	else if(ad_in == 2)
	{
		REG32WRITE(SYS->GPAMFP, ADC2, 1);   /* configure GPA2 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD2, 0);    /* configure GPA2 as input mode */
		channel = 2;
	}
	else if(ad_in == 3)
	{
		REG32WRITE(SYS->GPAMFP, ADC3, 1);   /* configure GPA3 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD3, 0);    /* configure GPA3 as input mode */
		channel = 3;
	}
	else if(ad_in == 4)
	{
		REG32WRITE(SYS->GPAMFP, ADC4, 1);   /* configure GPA4 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD4, 0);    /* configure GPA4 as input mode */
		channel = 4;
	}
	else if(ad_in == 5)
	{
		REG32WRITE(SYS->GPAMFP, ADC5, 1);   /* configure GPA5 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD5, 0);    /* configure GPA5 as input mode */
		channel = 5;
	}
	else if(ad_in == 6)
	{
		REG32WRITE(SYS->GPAMFP, ADC6, 1);   /* configure GPA6 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD6, 0);    /* configure GPA6 as input mode */
		channel = 6;
	}
	else if(ad_in == 7)
	{
		REG32WRITE(SYS->ALTMFP, PA7_S21, 0);
		REG32WRITE(SYS->GPAMFP, ADC7_SS21, 1);   /* configure GPA7 as analog input pin */
		REG32WRITE(GPIOA->PMD, PMD7, 0);         /* configure GPA7 as input mode */
		channel = 7;
	}

	/* Read to clear the channel state */
	adc_value = ADC->ADDR[channel].RSLT;

	/* Selects the AD pins to be sampled and converted. */
	REG32WRITE(ADC->ADCHER, CHEN, ((uint32_t)1 << channel));

	/* Start conversion now */
	REG32WRITE(ADC->ADCR, ADST, 1);

	/* Wait conversion complete */
	while(ADC->ADCR.ADST == 1);

	/* Get ADC result */
	adc_value = ADC->ADDR[channel].RSLT  ;
	
	return adc_value;
}

/***************************************************************************//**
 * @brief     Configure the ADC peripheral
 * @param[in] index : Configuration index, should be :
 *                      -COX_ADC_CFG_CLOCK : Conversion clock
 *                      -COX_ADC_CFG_BITS : Conversion Resolution, only 12
 * @param[in] arg : Configuration argument
 * @param[in] pre_arg : Return previous configuration argument
 * @return    COX_SUCCESS or COX_ERROR
*******************************************************************************/
static COX_Status NUC_ADC_Cfg (uint8_t index, uint32_t arg, uint32_t *pre_arg)
{
	uint32_t arg_bak;
	COX_Status result = COX_ERROR;

	switch(index) {
	case COX_ADC_CFG_CLOCK :
		arg_bak = NUC_ADC_SetRate(arg);
		result = COX_SUCCESS;  break;

	case COX_ADC_CFG_BITS :
		arg_bak = 12;            /* Only support 12-bit */
		if(arg != 12) break;
			result = COX_SUCCESS;  break;

	default : break;
	}

	if((result == COX_SUCCESS) && (pre_arg != COX_NULL)) {
		*pre_arg = arg_bak;
	}

	return result;
}

/***************************************************************************//**
 * Define NUC17xx COX ADC Peripheral Interface
*******************************************************************************/
COX_ADC_PI pi_adc = 
{
	NUC_ADC_Init,
	NUC_ADC_Read,
	NUC_ADC_Cfg,
};
