/***************************************************************************//**
 * @file      : nuc_exti.c
 * @brief     : NUC1xx EXTI Interface
 * @version   : V1.0
 * @date      : 21. August. 2010
 * @author    : CooCox
*******************************************************************************/
#include "cox_exti.h" 
#include "NUC1XX.H" 

#define PORT_OFFSET   0x40

static EXTI_Handler nuc_exti_handler = COX_NULL;     /* Interrupt Handler */

/***************************************************************************//**
 * @brief     External Interrupt 0 Handler
 * @param[in] None
 * @return    None
*******************************************************************************/
void EINT0_IRQHandler(void)     
{  
	/* EINT0 = GPB14. Clear the interrupt */
	GPIOB->ISRC  = (1UL << 14);
	if(nuc_exti_handler != COX_NULL)
	{
		nuc_exti_handler(COX_PIN(1, 14));
	}
} 

/***************************************************************************//**
 * @brief     External Interrupt 1 Handler
 * @param[in] None
 * @return    None
*******************************************************************************/
void EINT1_IRQHandler(void)     
{  
	/* EINT1 = GPB15. Clear the interrupt */
	GPIOB->ISRC  = (1UL << 15);
	if(nuc_exti_handler != COX_NULL)
	{
		nuc_exti_handler(COX_PIN(1, 15));
	}
} 

/***************************************************************************//**
 * @brief     Setup EXTI Interrupt Service Routine Call-Back Function
 * @param[in] isr : Pointer to the call-back function
 * @return    None
*******************************************************************************/
static void NUC_EXTI_Handler (EXTI_Handler handler)
{	
	nuc_exti_handler = handler;
	if(handler != COX_NULL)
	{
		NVIC_EnableIRQ(EINT0_IRQn);
		NVIC_EnableIRQ(EINT1_IRQn);
	}
	else
	{
		NVIC_DisableIRQ(EINT0_IRQn);
		NVIC_DisableIRQ(EINT1_IRQn);
	}
}

/***************************************************************************//**
 * @brief     Select Trigger mode
 * @param[in] pin : Specifie the EXTI line
 * @param[in] trigger : Specifie the trigger signal active mode, should be
 *              - COX_EXTI_NONE : None interrupt
 *              - COX_EXTI_EDGE_RISING : Rising edge interrupt
 *              - COX_EXTI_EDGE_FALLING : Falling edge interrupt
 *              - COX_EXTI_EDGE_RISING_FALLING : Both edge interrupt (Rising and Falling) 
 * @return    COX_SUCCESS : Success
 *            COX_ERROR : Error, parament error
*******************************************************************************/
COX_Status NUC_EXTI_Event (COX_PIN_Dev pin, uint8_t event)
{
	uint8_t port_num;
	uint8_t pin_num;
	COX_Status err = COX_ERROR;

	port_num = COX_PORT_NUM(pin); /* Get Port Number */
	pin_num  = COX_PIN_NUM(pin);  /* Get Pin Number  */

	if(port_num == 1)
	{
		if((pin_num == 14 )|| (pin_num == 15))
		{
		switch(event) {

			case COX_EXTI_EDGE_RISING:

				/* Select the edge trigger */
				outpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET) & (~(1 << pin_num )));
				/* Interrupt Enable  for Rising  Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << (pin_num + 16) ));
				/* Interrupt Dsiable for Falling Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) & (~(1 << pin_num ) ));

				err = COX_SUCCESS; break;

			case COX_EXTI_EDGE_FALLING:

				/* Select the edge trigger */
				outpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET) & (~(1 << pin_num )));
				/* Interrupt Enable  for Rising  Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) & (~(1 << (pin_num + 16 ) )));
				/* Interrupt Dsiable for Falling Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << pin_num ) );

				err = COX_SUCCESS; break;

			case COX_EXTI_EDGE_RISING_FALLING:

				/* Select the edge trigger */
				outpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET) & (~(1 << pin_num )));
				/* Interrupt Enable  for Rising  Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << (pin_num + 16 )) );
				/* Interrupt Dsiable for Falling Edge */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << pin_num )) ;

				err = COX_SUCCESS; break;


			case COX_EXTI_LEVEL_HIGH:
				/*  Select the level trigger */
				outpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET) | (1 << pin_num ) );
				/* Interrupt Enable  for high level */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << (pin_num + 16) ));
				/* Interrupt Dsiable for high level */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) & (~(1 << pin_num ) ));

			case COX_EXTI_LEVEL_LOW:
				/* Select the level trigger */
				outpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IMD + port_num * PORT_OFFSET) & (~(1 << pin_num )));
				/* Interrupt Enable  for low level */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << (pin_num + 16 )) );
				/* Interrupt Dsiable for low level */
				outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET) | (1 << pin_num )) ;

			case COX_EXTI_NONE :
			default : break;
			};
		}
	}
	return err;
}

/***************************************************************************//**
 * @brief     Initialize the EXTI line
 * @param[in] pin : Specifie the EXTI line
 * @param[in] trigger : Specifie the trigger signal active mode, should be
 *              - COX_EXTI_NONE : None interrupt
 *              - COX_EXTI_EDGE_RISING : Rising edge interrupt
 *              - COX_EXTI_EDGE_FALLING : Falling edge interrupt
 *              - COX_EXTI_EDGE_RISING_FALLING : Both edge interrupt (Rising and Falling) 
 * @return    COX_SUCCESS : Success
 *            COX_ERROR : Error, parament error
*******************************************************************************/
static COX_Status NUC_EXTI_Init (uint16_t pin)
{    
	COX_Status err = COX_ERROR;
	uint8_t port_num;
	uint8_t pin_num;
	uint32_t u32Reg ;

	port_num = COX_PORT_NUM(pin); /* Get Port Number */
	pin_num  = COX_PIN_NUM(pin);  /* Get Pin Number  */

	/* PB.14 PB.15 are used as external interrupt function*/
	if(port_num == 1)
	{
		if((pin_num == 14 )|| (pin_num == 15))
		{

			/* To initialize the multi-function pin's of the specified function. */
			REG32WRITE(SYS->GPBMFP, INT0, 1);
			REG32WRITE(SYS->GPBMFP, INT1_SS31, 1);
			REG32WRITE(SYS->ALTMFP, PB14_S31, 0);

			/* Set Direction as input */
			outpw((uint32_t)&GPIOA->PMD + port_num * PORT_OFFSET, (inpw((uint32_t)&GPIOA->PMD + port_num * PORT_OFFSET) & (~(3 << (pin_num * 2)))));

			/* Disable both rising/falling or both high level/low level interrupts */
			u32Reg = inpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET);
			u32Reg = u32Reg & (~((1UL << pin_num) | (1UL << (pin_num + 16))));
			outpw((uint32_t)&GPIOA->IEN + port_num * PORT_OFFSET , u32Reg);
			
			/* Clear Interrupt status */
			outpw((uint32_t)&GPIOA->ISRC + port_num * PORT_OFFSET , inpw((uint32_t)&GPIOA->ISRC + port_num * PORT_OFFSET) | (1 << pin_num ));

			err = COX_SUCCESS;
		}
	}


	return err;
}

/***************************************************************************//**
 * Define NUC1xx COX EXTI Interface based on GPIO
*******************************************************************************/
COX_EXTI_PI pi_exti = {
	NUC_EXTI_Handler,
	NUC_EXTI_Init,
	NUC_EXTI_Event,
};
