/*****************************************************************************//**
 * @file      : nuc_spi_master.c
 * @brief     : CoX SPI MASTER Peripheral Interface of NUC1xx
 * @version   : V1.0
 * @date      : 31. August. 2010
 * @author    : CooCox
 ******************************************************************************/
#include "NUC1xx.H"                
#include "nuc_spi_master.h"

static SPI_T * SPI_PORT[4]={SPI0, SPI1, SPI2, SPI3};

/*****************************************************************************//**
 * @brief     Get data size per transfer
 * @param[in] eSpiPort :select the spi 
 * @param[in] None
 * @return    Data size per transfer
*******************************************************************************/
static uint8_t NUC1xx_SPI_GetDataSize(uint8_t eSpiPort)
{
	uint8_t data_size = 0;
  
  	/* get the data size */
	if(SPI_PORT[eSpiPort]->CNTRL.TX_BIT_LEN == 0) 
	{
		data_size = 32;
	} 
	else 
	{
		data_size = SPI_PORT[eSpiPort]->CNTRL.TX_BIT_LEN & 0x1F;
	} 

	return data_size;
}

/*****************************************************************************//**
 * @brief     Setup SPI clock rate
 * @param[in] eSpiPort :select the spi 
 * @param[in] rate : Clock rate
 * @return    Previous clock rate
*******************************************************************************/
static uint32_t NUC1xx_SPI_SetRate (uint8_t eSpiPort,uint32_t rate)
{
	uint32_t spi_clk, spi_clk_bak;
	uint32_t prescale;
		  	
	spi_clk = (SPI_PORT[eSpiPort]->DIVIDER.DIVIDER) & 0xFFFF;
	spi_clk_bak = (12000000UL)/((spi_clk+1)<<2);

	/* Find closest clock to target clock */
	if(rate!=0)
	{
		prescale = (((12000000UL / rate) + 1) >> 1) - 1;
		if(prescale > 65535)
		    prescale = 65535;
		if(prescale < 1)
		    prescale = 1;
		/* Write to register */
		REG32WRITE(SPI_PORT[eSpiPort]->DIVIDER, DIVIDER, prescale);
	}

	return spi_clk_bak;
}

/*******************************************************************************
 * @brief     Initializes the SPIx peripheral
 * @param[in] eSpiPort :select the spi 
 * @param[in] mode : SPI mode, should be:eDRVSPI_MASTER
 * @param[in] rate : Clock rate
 * @return    Result, should be:
 *             -COX_SUCCESS
 *             -COX_ERROR
*******************************************************************************/
static COX_Status NUC1xx_SPI_Init (uint8_t eSpiPort,uint8_t mode,  uint32_t rate)
{
	uint32_t i,i32TimeOut;

	/* Unlock the protected registers */	
	UNLOCKREG();
	
	/* Enable the 12MHz oscillator oscillation */
	REG32WRITE(SYSCLK->PWRCON, XTL12M_EN, 1);
	
	/* Waiting for 12M Xtal stalble */
	for(i=0;i<10000;i++)
	
	/* HCLK clock source. 0: external 12MHz; 4:internal 22MHz RC oscillator */
	REG32WRITE(SYSCLK->CLKSEL0, HCLK_S, 0);
	
	LOCKREG();
	
	/* HCLK clock frequency = HCLK clock source / (HCLK_N + 1) */
	REG32WRITE(SYSCLK->CLKDIV, HCLK_N, 0);

  	/* init pins */
	if(eSpiPort == SPI_PORT0)
	{
		REG32WRITE(SYSCLK->APBCLK, SPI0_EN,       1);
		REG32WRITE(SYS->IPRSTC2, SPI0_RST,        1);
		REG32WRITE(SYS->IPRSTC2, SPI0_RST,        0);
		REG32WRITE(SYS->GPCMFP, SPI0_SS0_I2SLRCLK,1);
		REG32WRITE(SYS->ALTMFP, PC0_I2SLRCLK,     0);
		REG32WRITE(SYS->GPCMFP, SPI0_CLK_I2SBCLK, 1);
		REG32WRITE(SYS->ALTMFP, PC1_I2SBCLK,      0);
		REG32WRITE(SYS->GPCMFP, SPI0_MISO0_I2SDI, 1);
		REG32WRITE(SYS->ALTMFP, PC2_I2SDI,        0);
		REG32WRITE(SYS->GPCMFP, SPI0_MOSI0_I2SDO, 1);
		REG32WRITE(SYS->ALTMFP, PC3_I2SDO,        0);
		REG32WRITE(SYS->GPBMFP, TM2_SS01,         1);
		REG32WRITE(SYS->ALTMFP, PB10_S01,         1);		
	}
	else if(eSpiPort == SPI_PORT1)
	{
		REG32WRITE(SYSCLK->APBCLK, SPI1_EN, 1);
		REG32WRITE(SYS->IPRSTC2, SPI1_RST,  1);
		REG32WRITE(SYS->IPRSTC2, SPI1_RST,  0);
		REG32WRITE(SYS->GPCMFP, SPI1_SS0,   1);
		REG32WRITE(SYS->GPCMFP, SPI1_CLK,   1);
		REG32WRITE(SYS->GPCMFP, SPI1_MISO0, 1);
		REG32WRITE(SYS->GPCMFP, SPI1_MOSI0, 1);
		REG32WRITE(SYS->GPBMFP, TM1_SS11,   1);
		REG32WRITE(SYS->ALTMFP, PB9_S11,    1);
	}
	else if(eSpiPort == SPI_PORT2)
	{
		REG32WRITE(SYSCLK->APBCLK, SPI2_EN, 1);
		REG32WRITE(SYS->IPRSTC2, SPI2_RST,  1);
		REG32WRITE(SYS->IPRSTC2, SPI2_RST,  0);
		REG32WRITE(SYS->GPDMFP, SPI2_SS0,   1);
		REG32WRITE(SYS->GPDMFP, SPI2_CLK,   1);
		REG32WRITE(SYS->GPDMFP, SPI2_MISO0, 1);
		REG32WRITE(SYS->GPDMFP, SPI2_MOSI0, 1);
		REG32WRITE(SYS->GPAMFP, ADC7_SS21,  1);
		REG32WRITE(SYS->ALTMFP, PA7_S21,    1);
	}
	else
	{
		REG32WRITE(SYSCLK->APBCLK, SPI3_EN, 1);
		REG32WRITE(SYS->IPRSTC2, SPI3_RST,  1);
		REG32WRITE(SYS->IPRSTC2, SPI3_RST,  0);
		REG32WRITE(SYS->GPDMFP, SPI3_SS0,   1);
		REG32WRITE(SYS->GPDMFP, SPI3_CLK,   1);
		REG32WRITE(SYS->GPDMFP, SPI3_MISO0, 1);
		REG32WRITE(SYS->GPDMFP, SPI3_MOSI0, 1);
		REG32WRITE(SYS->GPBMFP, INT1_SS31,  1);
		REG32WRITE(SYS->ALTMFP, PB14_S31,   1);
	}

	/* select the spi mode*/
	if(mode==COX_SPI_MODE0)
	{
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  0);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  1);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  0);				
	}
	else if(mode==COX_SPI_MODE1)
	{
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  1);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  1);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  0);
	}
	else if(mode==COX_SPI_MODE2)
	{
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  0);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  0);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  1);
	}
	else
	{
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  1);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  0);
		REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  1);
	}

	/* MASTER */
	/* Set clock rate for SPI peripheral */
	NUC1xx_SPI_SetRate(eSpiPort,rate);
	/* Select the master mode */
	REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, SLAVE,  0);
	/* Set the active level of slave select. */
	REG32WRITE(SPI_PORT[eSpiPort]->SSR, SS_LVL,  0);

	/* Check busy*/
	i32TimeOut = 0x10000;
	while(SPI_PORT[eSpiPort]->CNTRL.GO_BUSY == 1)
	{
		if(i32TimeOut-- <= 0)
			return COX_ERROR;
	}

	return COX_SUCCESS;
}

/*****************************************************************************//**
 * @brief     Read write a data
 * @param[in] eSpiPort : SPI peripheral definition, should be SPI0SPI1SPI2SPI3.
 * @param[in] The data to Read wirte
 * @return    Data received
*******************************************************************************/
static uint32_t NUC1xx_SPI_ReadWrite (uint8_t eSpiPort,uint32_t send_data)                    
{   
	uint32_t read_data;
	
    while(SPI_PORT[eSpiPort]->CNTRL.GO_BUSY == 1);
	SPI_PORT[eSpiPort]->TX[0] = send_data;
	REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, GO_BUSY, 1); 
	
	while(SPI_PORT[eSpiPort]->CNTRL.GO_BUSY == 1) ;
	read_data = SPI_PORT[eSpiPort]->RX[0];
	 	 
	return read_data;
}

/*****************************************************************************//**
 * @brief     Transmit a block data through SPI peripheral
 * @param[in] eSpiPort :select the spi 
 * @param[in] wbuf : Pointer to transmit data
 * @param[in] wlen : Length of transfer data
 * @return    Actual Data length has been transferred
*******************************************************************************/
static uint32_t NUC1xx_SPI_Write (uint8_t eSpiPort,const void *wbuf,uint32_t wlen)
{
	uint32_t i;
	uint8_t data_size = NUC1xx_SPI_GetDataSize(eSpiPort);

	/* Move on only if NOT busy  */
	if(SPI_PORT[eSpiPort]->CNTRL.GO_BUSY==1)
		return COX_ERROR;

	for ( i = 0; i < wlen; i++ ) 
	{
		if(data_size<=8)
		{
			NUC1xx_SPI_ReadWrite(eSpiPort, ((uint8_t *)wbuf)[i]);
		}
		else if(data_size<=16 && data_size >8)
		{
			NUC1xx_SPI_ReadWrite(eSpiPort, ((uint16_t *)wbuf)[i]);
		}
		else{
			NUC1xx_SPI_ReadWrite(eSpiPort, ((uint32_t *)wbuf)[i]);
		}				 
	}

	return wlen; 
}

/*****************************************************************************//**
 * @brief     Receive a block data from SPI peripheral
 * @param[in] eSpiPort :select the spi 
 * @param[in] rbuf : Pointer to receive data
 * @param[in] wlen : Length of transfer data
 * @return    Actual Data length has been received
*******************************************************************************/
static uint32_t NUC1xx_SPI_Read (uint8_t eSpiPort,void *rbuf,uint32_t rlen )
{
	uint32_t i;
	uint8_t data_size = NUC1xx_SPI_GetDataSize(eSpiPort);

	for ( i = 0; i < rlen; i++ )
	{
		if(data_size<=8)
		{
			((uint8_t *)rbuf)[i] =NUC1xx_SPI_ReadWrite(eSpiPort, 0xFF);
		}
		else if(data_size<=16 && data_size >8)
		{
			((uint16_t *)rbuf)[i] =NUC1xx_SPI_ReadWrite(eSpiPort, 0xFFFF);
		}
		else{
			((uint32_t *)rbuf)[i] =NUC1xx_SPI_ReadWrite(eSpiPort, 0xFFFFFFFF);
		}
	}
	
	return rlen ;
}


/*****************************************************************************//**
 * @brief     Configure the SPI peripheral
 * @param[in] eSpiPort :select the spi 
 * @param[in] index : Specify the option
 * @param[in] arg : Argument of the specified option
 * @return    Result, may be :
 *              -COX_ERROR error occurred, parameter is not supported
 *              -COX_SUCCESS
*******************************************************************************/
static COX_Status NUC1xx_SPI_Cfg (uint8_t eSpiPort,uint8_t index, uint32_t arg, uint32_t *pre_arg)
{
	COX_Status result = COX_ERROR;
	uint32_t arg_bak;

	switch(index) {
	case COX_SPI_CFG_MODE:
		/* Transition modes */
		if(index==COX_SPI_MODE0)
		{
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  0);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  1);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  0);			
		}
		else if(index==COX_SPI_MODE1)
		{
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  1);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  1);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  0);
		}
		else if(index==COX_SPI_MODE2)
		{
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  0);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  0);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  1);
		}
		else
		{
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, CLKP,  1);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_NEG,  0);
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, RX_NEG,  1);
		}	
     
		result = COX_SUCCESS;
    break;

	case COX_SPI_CFG_RATE:
		arg_bak = NUC1xx_SPI_SetRate(eSpiPort,arg);
		result = COX_SUCCESS;
    break;

	case COX_SPI_CFG_BITS:
		arg_bak = NUC1xx_SPI_GetDataSize(eSpiPort);
   
		if(arg == 32) {
		/* Specify the 32 bit in a transaction */
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_BIT_LEN, 0);
			result = COX_SUCCESS;
		}
		else if((arg>0) && (arg<32)) {
			REG32WRITE(SPI_PORT[eSpiPort]->CNTRL, TX_BIT_LEN, arg);
			result = COX_SUCCESS;

		} 
		else {
			result = COX_ERROR;
		}
    break;

	default: break;
	}

	if((result == COX_SUCCESS) && (pre_arg != COX_NULL)) 
	{
		pre_arg = &arg_bak;
	}
	return result;
}

static uint32_t NUC1xx_SPI0_ReadWrite(uint32_t send_data) { return NUC1xx_SPI_ReadWrite(SPI_PORT0, send_data); }
static uint32_t NUC1xx_SPI1_ReadWrite(uint32_t send_data) { return NUC1xx_SPI_ReadWrite(SPI_PORT1, send_data); }
static uint32_t NUC1xx_SPI2_ReadWrite(uint32_t send_data) { return NUC1xx_SPI_ReadWrite(SPI_PORT2, send_data); }
static uint32_t NUC1xx_SPI3_ReadWrite(uint32_t send_data) { return NUC1xx_SPI_ReadWrite(SPI_PORT3, send_data); }

static uint32_t NUC1xx_SPI0_Write(const void *wbuf, uint32_t wlen ) { return NUC1xx_SPI_Write(SPI_PORT0, wbuf ,wlen); }
static uint32_t NUC1xx_SPI1_Write(const void *wbuf, uint32_t wlen ) { return NUC1xx_SPI_Write(SPI_PORT1, wbuf ,wlen); }
static uint32_t NUC1xx_SPI2_Write(const void *wbuf, uint32_t wlen ) { return NUC1xx_SPI_Write(SPI_PORT2, wbuf ,wlen); }
static uint32_t NUC1xx_SPI3_Write(const void *wbuf, uint32_t wlen ) { return NUC1xx_SPI_Write(SPI_PORT3, wbuf ,wlen); }

static uint32_t NUC1xx_SPI0_Read(void *rbuf, uint32_t rlen ) { return NUC1xx_SPI_Read(SPI_PORT0,rbuf, rlen); }
static uint32_t NUC1xx_SPI1_Read(void *rbuf, uint32_t rlen ) { return NUC1xx_SPI_Read(SPI_PORT1,rbuf, rlen); }
static uint32_t NUC1xx_SPI2_Read(void *rbuf, uint32_t rlen ) { return NUC1xx_SPI_Read(SPI_PORT2,rbuf, rlen); }
static uint32_t NUC1xx_SPI3_Read(void *rbuf, uint32_t rlen ) { return NUC1xx_SPI_Read(SPI_PORT3,rbuf, rlen); }

static COX_Status NUC1xx_SPI0_Cfg(uint8_t index, uint32_t arg, uint32_t *pre_arg) { return NUC1xx_SPI_Cfg(SPI_PORT0, index, arg, pre_arg); } 
static COX_Status NUC1xx_SPI1_Cfg(uint8_t index, uint32_t arg, uint32_t *pre_arg) { return NUC1xx_SPI_Cfg(SPI_PORT1, index, arg, pre_arg); } 
static COX_Status NUC1xx_SPI2_Cfg(uint8_t index, uint32_t arg, uint32_t *pre_arg) { return NUC1xx_SPI_Cfg(SPI_PORT2, index, arg, pre_arg); }
static COX_Status NUC1xx_SPI3_Cfg(uint8_t index, uint32_t arg, uint32_t *pre_arg) { return NUC1xx_SPI_Cfg(SPI_PORT3, index, arg, pre_arg); }

static COX_Status NUC1xx_SPI0_Init (uint8_t mode,  uint32_t rate)   { return NUC1xx_SPI_Init(SPI_PORT0, mode, rate);  }
static COX_Status NUC1xx_SPI1_Init (uint8_t mode,  uint32_t rate)   { return NUC1xx_SPI_Init(SPI_PORT1, mode, rate);  }
static COX_Status NUC1xx_SPI2_Init (uint8_t mode,  uint32_t rate)   { return NUC1xx_SPI_Init(SPI_PORT2, mode, rate);  }
static COX_Status NUC1xx_SPI3_Init (uint8_t mode,  uint32_t rate)   { return NUC1xx_SPI_Init(SPI_PORT3, mode, rate);  }

/*****************************************************************************//**
 * Define NUC1xx CoX SPI Peripheral Interface based on SPI0
*******************************************************************************/
COX_SPI_PI pi_spi0 = 
{
  NUC1xx_SPI0_Init,
  NUC1xx_SPI0_Write,
  NUC1xx_SPI0_Read,
  NUC1xx_SPI0_ReadWrite,
  NUC1xx_SPI0_Cfg,  
};

/*****************************************************************************//**
 * Define NUC1xx CoX SPI Peripheral Interface based on SPI1
*******************************************************************************/
COX_SPI_PI pi_spi1 = 
{
  NUC1xx_SPI1_Init,
  NUC1xx_SPI1_Write,
  NUC1xx_SPI1_Read,
  NUC1xx_SPI1_ReadWrite,
  NUC1xx_SPI1_Cfg,  
};

/*****************************************************************************//**
 * Define NUC1xx CoX SPI Peripheral Interface based on SPI2
*******************************************************************************/
COX_SPI_PI pi_spi2 = 
{
  NUC1xx_SPI2_Init,
  NUC1xx_SPI2_Write,
  NUC1xx_SPI2_Read,
  NUC1xx_SPI2_ReadWrite,
  NUC1xx_SPI2_Cfg,  
};

/*****************************************************************************//**
 * Define NUC1xx CoX SPI Peripheral Interface based on SPI3
*******************************************************************************/
COX_SPI_PI pi_spi3 = 
{
  NUC1xx_SPI3_Init,
  NUC1xx_SPI3_Write,
  NUC1xx_SPI3_Read,
  NUC1xx_SPI3_ReadWrite,
  NUC1xx_SPI3_Cfg,  
};
