/***************************************************************************//**
 * @file      : di_w25xx.c
 * @brief     : SPI Flash W25XX Driver Interface
 * @version   : 1.0
 * @date      : 30. Nov. 2010
 * @author    : CooCox
*******************************************************************************/
#include "di_w25xx.h"

/***************************************************************************//**
 * @brief     Enables the write access to the FLASH.
 * @param[in] dev : W25XX peripheral selected
 * @return    None
*******************************************************************************/
void W25XX_WriteEn(const W25XX_Dev *dev)
{
  uint8_t cmd;
  cmd = W25XX_CMD_WREN;

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)&cmd,1);
  dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     Init function
 * @param[in] dev : W25XX peripheral selected
 * @return    None
*******************************************************************************/
static void W25XX_Init(const W25XX_Dev *dev)
{
  dev->spi->Init(0,1000000);         	      /* Init the SPI Interface */
  dev->spi->Cfg(COX_SPI_CFG_BITS,8,COX_NULL); /* select the data length */

  dev->pio->Init(dev->cs_pin);    /* Init the Chip select pin */
  dev->pio->Dir(dev->cs_pin, 1);  /* Set the Chip select pin as output */
}

/***************************************************************************//**
 * @brief     Read ID
 * @param[in] dev : W25XX peripheral selected
 * @return    ID
*******************************************************************************/
uint32_t W25XX_ReadID(const W25XX_Dev *dev)
{
  uint16_t id;
  uint32_t id_buf[2];
  uint8_t adr_buf[4];

  adr_buf[0] = W25XX_CMD_RDID;
  adr_buf[1] = 0x00;
  adr_buf[2] = 0x00;
  adr_buf[3] = 0x00;

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)adr_buf,4);
  dev->spi->Read((uint8_t*)id_buf, 2);
  dev->pio->Out(dev->cs_pin, 1);

  id = (((uint32_t)id_buf[0] & 0x000000FF) << 8) | id_buf[1];		 // 0x00c22013
  id &= 0xFFFF;
  return id;
}

/***************************************************************************//**
 * @brief     Erases the entire FLASH
 * @param[in] dev : W25XX peripheral selected
 * @return    None
*******************************************************************************/
void W25XX_EraseChip(const W25XX_Dev *dev)
{
  uint8_t cmd;
  cmd = W25XX_CMD_CHIP_ERASE;

  W25XX_WriteEn(dev);

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)&cmd,1);
  dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     Erase the entire sector
 * @param[in] dev : W25XX peripheral selected
 * @param[in] sector : the sector to erase.
 * @return    None
*******************************************************************************/
void W25XX_EraseSector(const W25XX_Dev *dev, uint32_t adr)
{
  uint8_t adr_buf[4];

  adr_buf[0] = W25XX_CMD_SECTOR_ERASE;
  adr_buf[1] = (adr >> 16) & 0xFF;
  adr_buf[2] = (adr >>  8) & 0xFF;
  adr_buf[3] = (adr >>  0) & 0xFF;

  W25XX_WriteEn(dev);

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)adr_buf,4);
  dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     Erase the entire block
 * @param[in] dev W25XX peripheral selected
 * @param[in] block the block to erase.
 * @return    None
*******************************************************************************/
void W25XX_EraseBlock(const W25XX_Dev *dev, uint32_t adr)
{
  uint8_t adr_buf[4];

  adr_buf[0] = W25XX_CMD_BLOCK_ERASE;
  adr_buf[1] = (adr >> 16) & 0xFF;
  adr_buf[2] = (adr >>  8) & 0xFF;
  adr_buf[3] = (adr >>  0) & 0xFF;

  W25XX_WriteEn(dev);

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)adr_buf,4);
  dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     Get Flash Status
 * @param[in] dev : W25XX peripheral selected
 * @return    The FLASH status.
*******************************************************************************/
uint8_t W25XX_ReadStatus(const W25XX_Dev *dev)
{
  uint8_t cmd;
  uint8_t sr;
  cmd = W25XX_CMD_RDSR;

  dev->pio->Out(dev->cs_pin, 0);
  dev->spi->Write((uint8_t*)&cmd,1);
  sr = dev->spi->ReadWrite(0xFF);
  dev->pio->Out(dev->cs_pin, 1);

  return sr;
}

/***************************************************************************//**
 * @brief     Polls the status of the Write In Progress (WIP) flag in the
 *            FLASH's status register and loop until write opertaion
 *            has completed.
 * @param[in] dev : W25XX peripheral selected
 * @return    None
*******************************************************************************/
uint8_t W25XX_IsBusy(const W25XX_Dev *dev)
{
  return (W25XX_ReadStatus(dev) & 1);
}

/***************************************************************************//**
 * @brief     Polls the status of the Write In Progress (WIP) flag in the
 *            FLASH's status register and loop until write opertaion
 *            has completed.
 * @param[in] dev : W25XX peripheral selected
 * @return    None
*******************************************************************************/
void W25XX_WaitForReady(const W25XX_Dev *dev)
{

  do {
    W25XX_IsBusy(dev);
  } while((W25XX_IsBusy(dev) & 1) == 1);
}

/***************************************************************************//**
 * @brief    Writes more than one byte to the FLASH with a single WRITE
 *           cycle(Page WRITE sequence). The number of byte can't exceed
 *           the FLASH page size.
 * @param[in] dev : W25XX peripheral selected
 * @param[in] addr : FLASH's internal page to write to
 * @param[in] wbuf :  pointer to the buffer  containing the data to be
 *                    written to the FLASH.
 * @param[in] wlen : number of bytes to write to the FLASH,
 * @return    None
*******************************************************************************/
uint32_t W25XX_WritePage(const W25XX_Dev *dev, uint32_t addr, const uint8_t *wbuf, uint32_t wlen)
{
  uint8_t adr_buf[4];

  adr_buf[0] = W25XX_CMD_PROGRAM;
  adr_buf[1] = (addr >> 16) & 0xFF;
  adr_buf[2] = (addr >>  8) & 0xFF;
  adr_buf[3] = (addr >>  0) & 0xFF;

  W25XX_WriteEn(dev);

  dev->pio->Out(dev->cs_pin, 0);

  // Send write command and 3 bytes adress
  dev->spi->Write(adr_buf, 4);

  // Write data
  dev->spi->Write(wbuf, wlen);

  dev->pio->Out(dev->cs_pin, 1);

  return wlen;
}

/***************************************************************************//**
 * @brief    Writes more than one byte to the FLASH
 * @param[in] dev : W25XX peripheral selected
 * @param[in] addr : FLASH's internal page to write to
 * @param[in] wbuf :  pointer to the buffer  containing the data to be
 *                    written to the FLASH.
 * @param[in] wlen : number of bytes to write to the FLASH,
 * @return    None
*******************************************************************************/
uint32_t W25XX_Write(const W25XX_Dev *dev, uint32_t addr, const uint8_t *wbuf, uint32_t wlen)
{
  const uint8_t *WriteBuf;
  uint16_t FirstSize, LastSize, PageNum, i;
  uint32_t WriteAdr;

  FirstSize = addr % W25XX_PAGE_SIZE;

  WriteAdr = addr - FirstSize;
  FirstSize = W25XX_PAGE_SIZE - FirstSize;
  WriteBuf = wbuf;

  if(FirstSize < wlen) {
    PageNum  = (wlen - FirstSize) / W25XX_PAGE_SIZE;
    LastSize = (wlen - FirstSize) % W25XX_PAGE_SIZE;
  } else {
    FirstSize = wlen;
    PageNum = 0;
  }

  /* Write First Page */
  if(FirstSize != 0) {
    W25XX_WritePage (dev, addr, wbuf, FirstSize);
    WriteAdr += W25XX_PAGE_SIZE;
    WriteBuf += FirstSize;

    W25XX_WaitForReady(dev);
  }

  for(i=0; i<PageNum; i++) {
    W25XX_WritePage(dev, WriteAdr, WriteBuf, W25XX_PAGE_SIZE);
    WriteAdr += W25XX_PAGE_SIZE;
    WriteBuf += W25XX_PAGE_SIZE;

    W25XX_WaitForReady(dev);
  }

  /* Write Last Page */
  if(LastSize) {
    W25XX_WritePage(dev, WriteAdr, WriteBuf ,LastSize);
  }

  return wlen;
}

/***************************************************************************//**
 * @brief     Reads a block of data from the FLASH.
 * @param[in] dev : W25XX peripheral selected
 * @param[in] addr : FLASH's internal address to read from.
 * @param[in] rbuf : pointer to the buffer that receives the data read from the
                     FLASH.
 * @param[in] rlen : Number of bytes to read from the FLASH.
 * @return    Actual Data length has been read
*******************************************************************************/
uint32_t W25XX_Read(const W25XX_Dev *dev, uint32_t addr, uint8_t *rbuf, uint32_t rlen)
{
  uint8_t adr_buf[4];

  adr_buf[0] = W25XX_CMD_READ;
  adr_buf[1] = (addr >> 16) & 0xFF;
  adr_buf[2] = (addr >>  8) & 0xFF;
  adr_buf[3] = (addr >>  0) & 0xFF;

  // Send Read Command
  dev->pio->Out(dev->cs_pin, 0);

  // Send Read command and 3 bytes adress
  dev->spi->Write(adr_buf, 4);

  // Read data
  dev->spi->Read(rbuf, rlen);

  dev->pio->Out(dev->cs_pin, 1);

  return rlen;
}

/***************************************************************************//**
 * @brief     Get Flash Status
 * @param[in] dev : W25XX peripheral selected
 * @param[in] status : The new status
 * @return    None
*******************************************************************************/
void W25XX_WriteStatus(const W25XX_Dev *dev, uint8_t status)
{
  uint8_t adr_buf[2];

  adr_buf[0] = W25XX_CMD_WRSR;
  adr_buf[1] = status;

  // Send Read Command
  dev->pio->Out(dev->cs_pin, 0);

  // Write WriteSR command and the new status
  dev->spi->Write(adr_buf, 2);

  dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * Define SPI Flash W25X16 Driver Interface
*******************************************************************************/
W25XX_DI di_w25x16 =
{
  W25X16_PAGE_SIZE,
  W25X16_SECTOR_SIZE,
  W25X16_BLOCK_SIZE,
  W25X16_CHIP_SIZE,
  W25XX_Init,
  W25XX_ReadID,
  W25XX_Write,
  W25XX_Read,
  W25XX_EraseSector,
  W25XX_EraseBlock,
  W25XX_EraseChip,
  W25XX_IsBusy,
  W25XX_WaitForReady,
  W25XX_ReadStatus,
  W25XX_WriteStatus,
};

/***************************************************************************//**
 * Define SPI Flash W25X32 Driver Interface
*******************************************************************************/
W25XX_DI di_w25x32 =
{
  W25X32_PAGE_SIZE,
  W25X32_SECTOR_SIZE,
  W25X32_BLOCK_SIZE,
  W25X32_CHIP_SIZE,
  W25XX_Init,
  W25XX_ReadID,
  W25XX_Write,
  W25XX_Read,
  W25XX_EraseSector,
  W25XX_EraseBlock,
  W25XX_EraseChip,
  W25XX_IsBusy,
  W25XX_WaitForReady,
  W25XX_ReadStatus,
  W25XX_WriteStatus,
};

/***************************************************************************//**
 * Define SPI Flash W25X64 Driver Interface
*******************************************************************************/
W25XX_DI di_w25x64 =
{
  W25X64_PAGE_SIZE,
  W25X64_SECTOR_SIZE,
  W25X64_BLOCK_SIZE,
  W25X64_CHIP_SIZE,
  W25XX_Init,
  W25XX_ReadID,
  W25XX_Write,
  W25XX_Read,
  W25XX_EraseSector,
  W25XX_EraseBlock,
  W25XX_EraseChip,
  W25XX_IsBusy,
  W25XX_WaitForReady,
  W25XX_ReadStatus,
  W25XX_WriteStatus,
};

