/***************************************************************************//**
 * @file      : di_w25xx.h
 * @brief     : SPI Flash W25XX Driver Interface
 * @version   : 1.0
 * @date      : 30. Nov. 2010
 * @author    : CooCox
*******************************************************************************/
#ifndef __SPI_FLASH_W25XX_H
#define __SPI_FLASH_W25XX_H

#include "cox_pio.h"
#include "cox_spi.h"


/**
                 CS               CS  +----------+                            \n
      MCU_PIN*------- - - - - - ------+          |                            \n
                                      |          |                            \n
                                      |          |                            \n
  |----------+   CLK              SCK |          |                            \n
  |          +------- - - - - - ------+  W25XX   |                            \n
  |    SPI   |   MOSI             SI  |          |                            \n
  |          +------- - - - - - ------+          |                            \n
  |          |   MISO             SO  |          |                            \n
  |          +------- - - - - - ------+          |                            \n
  +----------+                        +----------+                            \n
                                                                              \n
                                                                              \n
W25XX Signal:                                                                \n
 -CS  : Chip Select                                                           \n
 -SCK : Serial Clock                                                          \n
 -SI  : Serial Data Input                                                     \n
 -SO  : Serial Data Output                                                    \n
                                                                              \n
W25X16 Features:                                                              \n
  Memory Size      : 2M Bytes                                                 \n
  Page Size        : 256                                                      \n
  Pages Per Sector : 16                                                       \n
  Sector Per Block : 16                                                       \n
  Blocks           : 32                                                       \n
  SPI Modes        : Modes 0 (0,0) or 3 (1,1)                                 \n
  Internal Adress  : 3 Bytes                                                  \n

W25X32 Features:                                                              \n
  Memory Size      : 4M Bytes                                                 \n
  Page Size        : 256                                                      \n
  Pages Per Sector : 16                                                       \n
  Sector Per Block : 16                                                       \n
  Blocks           : 64                                                       \n
  SPI Modes        : Modes 0 (0,0) or 3 (1,1)                                 \n
  Internal Adress  : 3 Bytes                                                  \n

W25X64 Features:                                                              \n
  Memory Size      : 8M Bytes                                                 \n
  Page Size        : 256                                                      \n
  Pages Per Sector : 16                                                       \n
  Sector Per Block : 16                                                       \n
  Blocks           : 128                                                       \n
  SPI Modes        : Modes 0 (0,0) or 3 (1,1)                                 \n
  Internal Adress  : 3 Bytes                                                  \n
                                                                              \n
Brief:                                                                        \n
  The W25X16/32/64 array is organized into 8,192/16,384/32,768 programmable   \n
  pages of 256-byteseach. Up to 256 bytes can be programmed at a time using   \n
  the Page Program instruction. Pages can be erased in groups of 16 (sector   \n
  erase), groups of 256 (block erase) or the entire chip (chip erase).The     \n
  W25X16/32/64 has 512/1024/2048 erasable sectors and 32/64/128 erasable      \n
  blocks respectively.The small 4KB sectors allow for greater flexibility     \n
  in applications that require data and parameterstorage.                     \n
*******************************************************************************/

/* INSTRUCTION CODES*/
#define W25XX_CMD_WREN          0x06 	  /*!< Set Write Enable Latch   */
#define W25XX_CMD_WRDI          0x04     /*!< Reset Write Enable Latch */
#define W25XX_CMD_RDSR          0x05     /*!< Read Status Register     */
#define W25XX_CMD_WRSR          0x01     /*!< Write Status Register    */
#define W25XX_CMD_READ          0x03     /*!< Read Data from Memory Array */
#define W25XX_CMD_FAST_READ     0x0B     /*!< Read Data from Memory Array (with dummy cycles) */
#define W25XX_CMD_PROGRAM       0x02     /*!< Page Program */
#define W25XX_CMD_SECTOR_ERASE  0x20     /*!< Erase One 4kbyte Sector in Memory Array */
#define W25XX_CMD_BLOCK_ERASE   0xD8     /*!< Erase One 64kbyte Block in Memory Array */
#define W25XX_CMD_CHIP_ERASE    0xC7     /*!< Erase All Memory Array */
#define W25XX_CMD_RDID          0x90     /*!< Read Manufacturer and Product ID */

#define W25XX_PAGE_SIZE        256UL     /* Page Size */

/* W25X16 SIZE */
#define W25X16_BLOCKS        32      /* Number of Block */
#define W25X16_SECTOR_PAGE   16      /* Number of Page per Sector */
#define W25X16_BLOCK_SECTOR  16      /* Number of Sector per Block */

#define W25X16_PAGE_SIZE      W25XX_PAGE_SIZE                          /* Page Size */
#define W25X16_SECTOR_SIZE   (W25X16_PAGE_SIZE * W25X16_SECTOR_PAGE)   /* Sector Size */
#define W25X16_BLOCK_SIZE    (W25X16_SECTOR_SIZE * W25X16_BLOCK_SECTOR)/* Block Size */
#define W25X16_CHIP_SIZE     (W25X16_BLOCKS * W25X16_BLOCK_SIZE)       /* Chip Size */

/* W25X32 SIZE */
#define W25X32_BLOCKS        64
#define W25X32_SECTOR_PAGE   16
#define W25X32_BLOCK_SECTOR  16

#define W25X32_PAGE_SIZE     W25XX_PAGE_SIZE
#define W25X32_SECTOR_SIZE   W25X16_SECTOR_SIZE
#define W25X32_BLOCK_SIZE    W25X16_BLOCK_SIZE
#define W25X32_CHIP_SIZE     (W25X32_BLOCKS * W25X32_BLOCK_SIZE)

/* W25X64 SIZE */
#define W25X64_BLOCKS        128
#define W25X64_SECTOR_PAGE   16
#define W25X64_BLOCK_SECTOR  16

#define W25X64_PAGE_SIZE     W25XX_PAGE_SIZE
#define W25X64_SECTOR_SIZE   W25X16_SECTOR_SIZE
#define W25X64_BLOCK_SIZE    W25X16_BLOCK_SIZE
#define W25X64_CHIP_SIZE    (W25X64_BLOCKS * W25X64_BLOCK_SIZE)

/** W25XX Device Definition
*******************************************************************************/
typedef struct
{
  uint16_t cs_pin;             /*!< Chip Select  */
  COX_PIO_PI  *pio;            /*!< PIO Peripheral Interface Used */
  COX_SPI_PI  *spi;            /*!< SPI Peripheral Interface Used */
} W25XX_Dev;

/** W25XX Interface Definition
*******************************************************************************/
typedef struct
{
  uint32_t PageSize;
  uint32_t SectorSize;
  uint32_t BlockSize;
  uint32_t ChipSize;
  void     (*Init)         (const W25XX_Dev *dev);
  uint32_t (*ReadID)       (const W25XX_Dev *dev);
  uint32_t (*Write)        (const W25XX_Dev *dev, uint32_t addr, const uint8_t *wbuf, uint32_t wlen);
  uint32_t (*Read)         (const W25XX_Dev *dev, uint32_t addr,       uint8_t *rbuf, uint32_t rlen);
  void     (*EraseSector)  (const W25XX_Dev *dev, uint32_t addr);
  void     (*EraseBlock)   (const W25XX_Dev *dev, uint32_t addr);
  void     (*EraseChip)    (const W25XX_Dev *dev);
  uint8_t  (*IsBusy)       (const W25XX_Dev *dev);
  void     (*WaitForReady) (const W25XX_Dev *dev);
  uint8_t  (*ReadStatus)   (const W25XX_Dev *dev);
  void     (*WriteStatus)  (const W25XX_Dev *dev, uint8_t status);

} W25XX_DI_Def;

typedef const W25XX_DI_Def W25XX_DI;

/** Define SPI Flash W25X16 Driver Interface
*******************************************************************************/
extern W25XX_DI di_w25x16;

/** Define SPI Flash W25X32 Driver Interface
*******************************************************************************/
extern W25XX_DI di_w25x32;

/** Define SPI Flash W25X64 Driver Interface
*******************************************************************************/
extern W25XX_DI di_w25x64;

#endif
