/***************************************************************************//**
 * @file      : WAU8822.c
 * @brief     : WAU8822 Driver Interface
 * @version   : 1.0
 * @date      : 18. Dec. 2010
 * @author    : CooCox
*******************************************************************************/
#include "WAU8822.h"

COX_Status Temp_Status = COX_ERROR;

/***************************************************************************//**
 * @brief      Initializes the WAU8822.
 * @param[in]  dev : WAU8822 device selected.
 * @return     None
*******************************************************************************/
COX_Status WAU_Init (const WAU_Dev* dev)
{
  uint32_t i = 0;

  dev->i2c->Init(100000);

  Temp_Status = WAU_Write_Reg (dev, 0, 0x000);

  for(i = 0; i < 200; i++);	                        /*!< Delay                                                    */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 1, 0x02F);
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 2, 0x1B3);		  /*!< Enable L/R Headphone, ADC Mix/Boost, ADC                 */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 3, 0x00F);		  /*!< Enable L/R main mixer, DAC                               */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 4, 0x010);	    /*!< 16-bit word length, I2S format, Stereo                   */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 5, 0x000);		  /*!< Companding control and loop back mode (all disable)      */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 6, 0x1AD);		  /*!< Divide by 6, 16K                                         */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 7, 0x006);	    /*!< 16K for internal filter cofficients                      */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 10, 0x008);	    /*!< DAC softmute is disabled, DAC oversampling rate is 128x  */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 14, 0x108);		  /*!< ADC HP filter is disabled, ADC oversampling rate is 128x */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 15, 0x1EF);		  /*!< ADC left digital volume control                          */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 16, 0x1EF);		  /*!< ADC right digital volume control                         */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 44, 0x000);	    /*!< LLIN/RLIN is not connected to PGA                        */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 47, 0x050);		  /*!< LLIN connected, and its Gain value                       */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 48, 0x050);	    /*!< RLIN connected, and its Gain value                       */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 50, 0x001);		  /*!< Left DAC connected to LMIX                               */
  while (Temp_Status == COX_ERROR);

  Temp_Status = WAU_Write_Reg (dev, 51, 0x001);	    /*!< Right DAC connected to RMIX                              */
  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     3D Stereo Enhancement effect depth control.
 * @param[in] dev          : WAU8822 device selected
 * @param[in] WAU_3D_Depth : Setting becomes active when allowed by zero crossing and/or update bit features.
 *                          - WAU_3D_Depth = 0x00 (0.0%  effect)
 *					        - WAU_3D_Depth = 0x01 (6.67% effect)
 *							- WAU_3D_Depth = 0x02 (13.3% effect)
 *					        - effect depth varies by 6.67% per binary bit value C
 *					        - WAU_3D_Depth = 0x0E (93.3% effect)
 *					        - WAU_3D_Depth = 0x0F (100%  effect, maximum effect)
 * @return    None
*******************************************************************************/
COX_Status WAU_3D_Control (const WAU_Dev* dev, uint16_t WAU_3D_Depth)
{
  uint8_t addr = 0x29;

  Temp_Status = WAU_Write_Reg (dev, addr, WAU_3D_Depth);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Left channel headphone output volume control setting.
 * @param[in] dev    : WAU8822 device selected
 * @param[in] volume : Setting becomes active when allowed by zero crossing and/or update bit features.
 *                     - volume = 0x00 (-57dB)
 *					   - volume = 0x01 (-56dB)
 *					   - volume changes in 1.0dB steps per binary bit value C
 *					   - volume = 0x3E (+5dB)
 *					   - volume = 0x3F (+6dB)
 * @param[in] mute   : Left headphone output mute control :
 *                     - WAU_NORMAL
 *                     - WAU_MUTE
 * @return    None
*******************************************************************************/
COX_Status WAU_LHP_volume (const WAU_Dev* dev, uint8_t volume, uint8_t mute)
{
  uint8_t addr = 0x34;						 /*!< WAU8822 register address                                         */

  uint16_t value = volume;

  if (WAU_NORMAL == mute) {
     value &= ~(0x0040); 					 /*!< headphone output not muted, normal operation                     */
  }  else {
  	 value |= 0x0040; 					   /*!< headphone in muted condition not connected to LMIX output stage  */
  }

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Right channel headphone output volume control setting.
 * @param[in] dev    : WAU8822 device selected
 * @param[in] volume : Setting becomes active when allowed by zero crossing and/or update bit features.
 *                     - volume = 0x00 (-57dB)
 *					   - volume = 0x01 (-56dB)
 *					   - volume changes in 1.0dB steps per binary bit value C
 *					   - volume = 0x3E (+5dB)
 *					   - volume = 0x3F (+6dB)
 * @param[in] mute   : Right headphone output mute control :
 *                     - mute = WAU_NORMAL
 *                     - mute = WAU_MUTE
 * @return    None
*******************************************************************************/
COX_Status WAU_RHP_volume (const WAU_Dev* dev, uint8_t volume, uint8_t mute)
{
  uint8_t addr = 0x35;						 /*!< WAU8822 register address                                          */

  uint16_t value = volume;

  if (1 == mute) {
     value &= ~(0x0040); 					 /*!< headphone output not muted, normal operation                      */
  }  else {
  	 value |= 0x0040; 					   /*!< headphone in muted condition not connected to RMIX output stage   */
  }

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Left loudspeaker output volume control setting.
 * @param[in] dev    : WAU8822 device selected
 * @param[in] volume : Setting becomes active when allowed by zero crossing and/or update bit features.
 *                     - volume = 0x00 (-57dB)
 *					   - volume = 0x01 (-56dB)
 *					   - volume changes in 1.0dB steps per binary bit value C
 *					   - volume = 0x3E (+5dB)
 *					   - volume = 0x3F (+6dB)
 * @param[in] mute   : Left loudspeaker LSPKOUT output mute control :
 *                     - mute = WAU_NORMAL
 *                     - mute = WAU_MUTE
 * @return    None
*******************************************************************************/
COX_Status WAU_LSPKOUT_volume (const WAU_Dev* dev, uint8_t volume, uint8_t mute)
{
  uint8_t addr = 0x36;						  /*!< WAU8822 register address                                         */

  uint16_t value = volume;

  if (1 == mute) {
     value &= ~(0x0040); 					 /*!< loudspeaker output not muted, normal operation                    */
  }  else {
  	 value |= 0x0040; 					   /*!< loudspeaker in muted condition                                    */
  }

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

/***************************************************************************//**
 * @brief     Right loudspeaker output volume control setting.
 * @param[in] dev    : WAU8822 device selected
 * @param[in] volume : Setting becomes active when allowed by zero crossing and/or update bit features.
 *                     - volume = 0x00 (-57dB)
 *					   - volume = 0x01 (-56dB)
 *					   - volume changes in 1.0dB steps per binary bit value C
 *					   - volume = 0x3E (+5dB)
 *					   - volume = 0x3F (+6dB)
 * @param[in] mute   : Right loudspeaker LSPKOUT output mute control :
 *                     - mute = WAU_NORMAL
 *                     - mute = WAU_MUTE
 * @return    None
*******************************************************************************/
COX_Status WAU_RSPKOUT_volume (const WAU_Dev* dev, uint8_t volume, uint8_t mute)
{
  uint8_t addr = 0x37;						  /*!< WAU8822 register address                                         */

  uint16_t value = volume;

  if (1 == mute) {
     value &= ~(0x0040); 					  /*!< loudspeaker output not muted, normal operation                    */
  }  else {
  	 value |= 0x0040; 					    /*!< loudspeaker in muted condition                                    */
  }

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     EQ1 low cutoff
 * @param[in] dev   : WAU8822 device selected
 * @param[in] EQ1CF : Equalizer band 1 low pass -3dB cut-off frequency selection :
 *                     - EQ1CF = 0x00 (80Hz)
 *					   - EQ1CF = 0x01 (105Hz)
 *					   - EQ1CF = 0x02 (135Hz)
 *					   - EQ1CF = 0x03 (175Hz)
 * @param[in] EQ1GC : EQ Band 1 digital gain control :
 *                     - EQ1GC = 0x0C(0.0dB) default unity gain value
 *                     - EQ1GC = 0x00(+12dB)
 *                     - EQ1GC = 0x01(+11dB)
 *                     - all intermediate 1.0dB step values through minimum gain -
 *         			   - EQ1GC = 0x18(-12dB)
 * 					   - 0x19 and larger values are reserved
 * @return    None
*******************************************************************************/
COX_Status WAU_EQ1 (const WAU_Dev* dev, uint16_t EQ1CF, uint16_t EQ1GC)
{
  uint8_t  addr  = 0x12;				    /*!< WAU8822 register address                                            */

  uint16_t value = EQ1CF << 5;

  value |= EQ1GC;

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     EQ2 - peak 1
 * @param[in] dev   : WAU8822 device selected
 * @param[in] EQ2CF : Equalizer Band 2 center frequency selection :
 *                     - EQ2CF = 0x00 (230Hz)
 *					   - EQ2CF = 0x01 (300Hz)
 *					   - EQ2CF = 0x02 (385Hz)
 *					   - EQ2CF = 0x03 (500Hz)
 * @param[in] EQ2GC : EQ Band 2 digital gain control :
 *                     - EQ2GC = 0x0C(0.0dB) default unity gain value
 *                     - EQ2GC = 0x00(+12dB)
 *                     - EQ2GC = 0x01(+11dB)
 *                     - all intermediate 1.0dB step values through minimum gain -
 *         			   - EQ2GC = 0x18(-12dB)
 * 					   - 0x19 and larger values are reserved
 * @return    None
*******************************************************************************/
COX_Status WAU_EQ2 (const WAU_Dev* dev, uint16_t EQ2CF, uint16_t EQ2GC)
{
  uint8_t  addr  = 0x13;				 /*!< WAU8822 register address                                    */

  uint16_t value = EQ2CF << 5;

  value |= EQ2GC;

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     EQ3 - peak 2
 * @param[in] dev   : WAU8822 device selected
 * @param[in] EQ3CF : Equalizer Band 3 center frequency selection :
 *                     - EQ3CF = 0x00 (650Hz)
 *					   - EQ3CF = 0x01 (850Hz)
 *					   - EQ3CF = 0x02 (1.1kHz)
 *					   - EQ3CF = 0x03 (1.4kHz)
 * @param[in] EQ3GC : EQ Band 3 digital gain control :
 *                     - EQ3GC = 0x0C (0.0dB) default unity gain value
 *                     - EQ3GC = 0x00 (+12dB)
 *                     - EQ3GC = 0x01 (+11dB)
 *                     - all intermediate 1.0dB step values through minimum gain -
 *         			   - EQ3GC = 0x18 (-12dB)
 * 					   - 0x19 and larger values are reserved
 * @return    None
*******************************************************************************/
COX_Status WAU_EQ3 (const WAU_Dev* dev, uint16_t EQ3CF, uint16_t EQ3GC)
{
  uint8_t  addr  = 0x14;			     /*!< WAU8822 register address                                     */

  uint16_t value = EQ3CF << 5;

  value |= EQ3GC;

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     EQ4 - peak 3
 * @param[in] dev   : WAU8822 device selected
 * @param[in] EQ4CF : Equalizer Band 4 center frequency selection :
 *                     - EQ4CF = 0x00 (1.8kHz)
 *					   - EQ4CF = 0x01 (2.4kHz)
 *					   - EQ4CF = 0x02 (3.2kHz)
 *					   - EQ4CF = 0x03 (4.1kHz)
 * @param[in] EQ4GC : EQ Band 4 digital gain control :
 *                     - EQ4GC = 0x0C (0.0dB) default unity gain value
 *                     - EQ4GC = 0x00 (+12dB)
 *                     - EQ4GC = 0x01 (+11dB)
 *                     - all intermediate 1.0dB step values through minimum gain -
 *         			   - EQ4GC = 0x18 (-12dB)
 * 					   - 0x19 and larger values are reserved
 * @return    None
*******************************************************************************/
COX_Status WAU_EQ4 (const WAU_Dev* dev, uint16_t EQ4CF, uint16_t EQ4GC)
{
  uint8_t  addr  = 0x15;			   /*!< WAU8822 register address                                      */

  uint16_t value = EQ4CF << 5;

  value |= EQ4GC;

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     EQ5 - peak 3
 * @param[in] dev   : WAU8822 device selected
 * @param[in] EQ5CF : Equalizer Band 5 high pass -3dB cut-off frequency selection :
 *                     - EQ5CF = 0x00 (5.3kHz)
 *					   - EQ5CF = 0x01 (6.9kHz)
 *					   - EQ5CF = 0x02 (9.0kHz)
 *					   - EQ5CF = 0x03 (11.7kHz)
 * @param[in] EQ5GC : EQ Band 5 digital gain control :
 *                     - EQ5GC = 0x0C (0.0dB) default unity gain value
 *                     - EQ5GC = 0x00 (+12dB)
 *                     - EQ5GC = 0x01 (+11dB)
 *                     - all intermediate 1.0dB step values through minimum gain -
 *         			   - EQ5GC = 0x18 (-12dB)
 * 					   - 0x19 and larger values are reserved
 * @return    None
*******************************************************************************/
COX_Status WAU_EQ5 (const WAU_Dev* dev, uint16_t EQ5CF, uint16_t EQ5GC)
{
  uint8_t  addr  = 0x16;			   /*!< WAU8822 register address                                     */

  uint16_t value = EQ5CF << 5;

  value |= EQ5GC;

  Temp_Status = WAU_Write_Reg (dev, addr, value);

  while (Temp_Status == COX_ERROR);

  return COX_SUCCESS;
}

 /***************************************************************************//**
 * @brief     Write the register of the WAU8822 device.
 * @param[in] dev   : WAU8822 device selected.
 * @param[in] reg   : Register selected, it should be :
 *                     - reg = 0x00 (Power Management 1 register)
 *					   - reg = 0x01 (Power Management 2 register)
 *					   - reg = 0x02 - 0x3F (Relate to WAU8822 Design Guide)
 * @param[in] value : Register value
 * @return    None
*******************************************************************************/
COX_Status WAU_Write_Reg (const WAU_Dev* dev, uint8_t reg, uint16_t value)
{
  I2C_Transfer_Cfg cfg;
  uint8_t Tx_Data[2];

  Tx_Data[0] = (uint8_t)((reg << 1) | (value >> 8));
  Tx_Data[1] = value & 0xFF;

  cfg.slave = dev->adr;
  cfg.rbuf  = COX_NULL;
  cfg.rlen  = 0;
  cfg.wbuf  = Tx_Data;
  cfg.wlen  = 2;

  if(dev->i2c->Transfer(&cfg) == COX_SUCCESS) {
	 return COX_SUCCESS;
  } else {
	 return COX_ERROR;
  }
}

 /***************************************************************************//**
 * @brief     Read the register of the WAU8822 device.
 * @param[in] dev   : WAU8822 device selected.
 * @param[in] reg   : Register selected, it should be :
 *                     - reg = 0x00 (Power Management 1 register)
 *					   - reg = 0x01 (Power Management 2 register)
 *					   - reg = 0x02 - 0x3F (Relate to WAU8822 Design Guide)
 * @return    Register value
*******************************************************************************/
uint16_t WAU_Read_Reg (const WAU_Dev* dev, uint8_t reg)
{
  I2C_Transfer_Cfg cfg;
  uint8_t tmp_tra = reg << 1;
  uint8_t Rx_Data[2];

  cfg.slave = dev->adr;
  cfg.wbuf = &tmp_tra;
  cfg.wlen = 1;
  cfg.rlen = 2;
  cfg.rbuf = Rx_Data;

  if(dev->i2c->Transfer(&cfg) == COX_SUCCESS) {
         return (((uint16_t)Rx_Data[0] << 8) | Rx_Data[1]);
  } else {
	  return COX_ERROR;
  }


}
/***************************************************************************//**
 * Defined WAU8822 device interface
*******************************************************************************/
const WAU_AUDIO_DI di_wau = {
  WAU_Init,            /*!< Initializes the WAU8822                               */
  WAU_3D_Control,      /*!< 3D Stereo Enhancement effect depth control            */
  WAU_LHP_volume,      /*!< Left channel headphone output volume control setting   */
  WAU_RHP_volume,      /*!< Right channel headphone output volume control setting */
  WAU_LSPKOUT_volume,  /*!< Left loudspeaker output volume control setting        */
  WAU_RSPKOUT_volume,  /*!< Right loudspeaker output volume control setting       */
  WAU_EQ1,             /*!< Equalizer Band 1                                      */
  WAU_EQ2,             /*!< Equalizer Band 2                                      */
  WAU_EQ3,             /*!< Equalizer Band 3                                      */
  WAU_EQ4,             /*!< Equalizer Band 4                                      */
  WAU_EQ5,             /*!< Equalizer Band 5                                      */
  WAU_Write_Reg,       /*!< Write the register of the WAU8822 device              */
  WAU_Read_Reg,	       /*!< Read the register of the WAU8822 device               */
};




