/**************************************************************************//**
 * @file      : di_UC1601.c
 * @brief     : SPI TEXT LCD HT1621 Driver Interface
 * @version	  : 1.0
 * @date      : 27 NUV. 2010
 * @author    : CooCox
******************************************************************************/
#include "di_uc1601.h"

extern  char Ascii[];

/***************************************************************************//**
 * @brief	   Init the LCD device
 * @param[in]  SPI interface selected
 * @return     None
*******************************************************************************/
void SPI_UC1601_Init(const UC1601_Dev *dev)
{
	dev->spi->Init(COX_SPI_MODE1, 1500000);
	dev->spi->Cfg(COX_SPI_CFG_BITS,9,COX_NULL); /* 9-serial mode */
		
	dev->pio->Init(dev->cs_pin);    /* Init the Chip select pin */
	dev->pio->Dir(dev->cs_pin, 1);  /* Set the Chip select pin as output */

	dev->pio->Out(dev->cs_pin, 0);	/* Select the slaver */
	dev->spi->ReadWrite(0xEB);  	/* Set BR */
	dev->pio->Out(dev->cs_pin, 1);	/* de-active the slaver */

	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0x81);		/* Set PM */
	dev->spi->ReadWrite(0xA0);
	dev->pio->Out(dev->cs_pin, 1);

	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0xC0);		/* outp32 */
	dev->pio->Out(dev->cs_pin, 1);

	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0xAF); 		/* Set Display Enable */
	dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     Send a data to the device
 * @param[in] spi  : SPI interface selected
 * @param[in] data : The data to send
 * @return    None
*******************************************************************************/
void SPI_UC1601_Write(const UC1601_Dev *dev, unsigned char data)
{
	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0x100 | data); /* send the serial-9 data */
	dev->pio->Out(dev->cs_pin, 1);
}


/***************************************************************************//**
 * @brief     initial start address
 * @param[in] spi  : SPI interface selected
 * @param[in] PA : The X axis position
 * @param[in] CA : The Y axis position
 * @return    None
*******************************************************************************/
void SPI_UC1601_SetPACA(const UC1601_Dev *dev, unsigned char PA, unsigned char CA)
{
	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0xB0 | PA); 	         /* Set PA */
	dev->pio->Out(dev->cs_pin, 1);

	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0x10 |((CA>>4)&(0xF)));  /* Set CA MSB */
	dev->pio->Out(dev->cs_pin, 1);

	dev->pio->Out(dev->cs_pin, 0);
	dev->spi->ReadWrite(0x00 | (CA & 0xF));      /* Set CA LSB */
	dev->pio->Out(dev->cs_pin, 1);
}

/***************************************************************************//**
 * @brief     send the data to the address
 * @param[in] spi  : SPI interface selected
 * @param[in] x : The line of the X axis
 * @param[in] y : The position of the line
 * @param[in] ascii_word : The word's ascii data
 * @return    None
*******************************************************************************/
void SPI_UC1601_Show(const UC1601_Dev *dev, unsigned char x, unsigned char y,unsigned char ascii_word)
{
	int i=0,k=0;
	unsigned char temp;

	k=(ascii_word-32)*16;
	  
	for(i=0;i<8;i++)
	{
		SPI_UC1601_SetPACA(dev,(x*2),(129-(y*8)-i));
		temp=Ascii[k+i];
		SPI_UC1601_Write(dev,temp);
	}

	for(i=0;i<8;i++)
	{
		SPI_UC1601_SetPACA(dev,(x*2)+1,(129-(y*8)-i));
		temp=Ascii[k+i+8];
		SPI_UC1601_Write(dev,temp);
	}
}

/***************************************************************************//**
 * @brief     show the word on LCD
 * @param[in] spi  : SPI interface selected
 * @param[in] line : The line of the X axis
 * @param[in] str : the word which will be showed
 * @return    None
*******************************************************************************/
void SPI_UC1601_print(const UC1601_Dev *dev, unsigned char line, char *str)
{
	int i=0;
	do{
		SPI_UC1601_Show(dev,line,i,*str++);
		i++;
		if(i>15)
			break;
    }
	while(*str!='\0');
}

/***************************************************************************//**
 * @brief     clear all pannal
 * @param[in] spi  : SPI interface selected
 * @return    None
*******************************************************************************/
void SPI_UC1601_clr(const UC1601_Dev *dev)
{
	int i=0;

	SPI_UC1601_SetPACA(dev,0x0, 0x0);  /*CLEAR ALL PANNAL*/

	for (i = 0; i < 132 *8; i++)
	{
		SPI_UC1601_Write(dev,0x00);
	}

	SPI_UC1601_Write(dev,0x0f);
}

/***************************************************************************//**
 * Define SPI TEXT LCD UC1601 Device Driver Interface
*******************************************************************************/
const SLCD_SPI_DI di_uc1601 = {
	SPI_UC1601_Init,
	SPI_UC1601_SetPACA,
	SPI_UC1601_Write,
	SPI_UC1601_Show,
	SPI_UC1601_clr,
	SPI_UC1601_print,
};
