/********************************************************************************************************//**
 * @file    : DrvSYS.c
 * @brief   : Contains all functions support for SYS 
 * @version : 1.1.3
 * @author  : Nuvoton
 * @history :
 *           <author>      <time>     <version>      <desc>
 *           CooCox      2010.10.18    1.1.3     Modify the problem cause by volatile bit-field in the GCC according
 *                                               to http://www.codesourcery.com/archives/arm-gnu/msg03675.html
 *
 *           Nuvoton      2010         1.1.2     Initial version
 ************************************************************************************************************
 * Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved
************************************************************************************************************/
//#include <stdio.h>
#include "NUC1xx.h"
#include "DrvSYS.h"

/********************************************************************************************************//**
 * Global file scope (static) variables
************************************************************************************************************/
static uint32_t s_u32ExtClockKHz = 12000;

static BOD_CALLBACK BOD_callbackFn;
static PWRWU_CALLBACK PWRWU_callbackFn;

/********************************************************************************************************//**
 * @brief     The function is used to get the DrvSYS version number
 * @param[in] None
 * @return    The DrvSYS version number
************************************************************************************************************/
uint32_t DrvSYS_GetVersion(void)
{
    return DRVSYS_VERSION_NUM;
}

/********************************************************************************************************//**
 * @brief     The function is used to Read Product ID
 * @param[in] None
 * @return    Product ID
************************************************************************************************************/
uint32_t DrvSYS_ReadProductID(void)
{   
    return SYS->PDID;
}

/********************************************************************************************************//**
 * @brief     The function is used to Unlock protected registers
 * @param[in] None
 * @return    -  0      Success 
 *            -  <0     Failed
************************************************************************************************************/
int32_t DrvSYS_UnlockKeyAddr(void)
{
    uint32_t lockbit;
    
    SYS->REGLOCK = 0x59;
    SYS->REGLOCK = 0x16;
    SYS->REGLOCK = 0x88;

    lockbit = SYS->REGLOCK & 0x01;
    return (lockbit)? 0:E_DRVSYS_ERR_UNLOCK_FAIL;
} 
 
/********************************************************************************************************//**
 * @brief     The function is used to Lock protected registers
 * @param[in] None
 * @return    -  0      Success 
 *            -  <0     Failed
************************************************************************************************************/
int32_t DrvSYS_LockKeyAddr(void)
{
    uint32_t lockbit;
    
    SYS->REGLOCK = 0x00;

    lockbit = SYS->REGLOCK & 0x01;
    return (lockbit)? E_DRVSYS_ERR_LOCK_FAIL:0;
}

/********************************************************************************************************//**
 * @brief     The function is used to Get reset source from last operation
 * @param[in] None
 * @return    Reset source  
************************************************************************************************************/
uint32_t DrvSYS_GetRstSrc(void)
{
    return *((__IO uint32_t *) &SYS->RSTSRC);
}

/********************************************************************************************************//**
 * @brief     The function is used to Clear reset source by write 1 
 * @param[in] u32Src  :   The corresponding bit of reset source
 * @return    0
************************************************************************************************************/
uint32_t DrvSYS_ClearRstSrc(uint32_t u32Src)
{
    *((__IO uint32_t *) &SYS->RSTSRC) = u32Src;
    return 0;
}


/********************************************************************************************************//**
 * @brief     The function is used to  Reset coresponding IP 
 * @param[in]  eIpRst               E_SYS_GPIO_RST  / E_SYS_TMR0_RST / E_SYS_TMR1_RST / E_SYS_TMR2_RST     
 *                                  E_SYS_TMR3_RST  / E_SYS_I2C0_RST / E_SYS_I2C1_RST / E_SYS_SPI0_RST  
 *                                  E_SYS_SPI1_RST  / E_SYS_SPI2_RST / E_SYS_SPI3_RST / E_SYS_UART0_RST  
 *                                  E_SYS_UART1_RST / E_SYS_UART2_RST/ E_SYS_PWM03_RST/ E_SYS_PWM47_RST    
 *                                  E_SYS_ACMP_RST  / E_SYS_PS2_RST  / E_SYS_CAN0_RST / E_SYS_USBD_RST     
 *                                  E_SYS_ADC_RST   / E_SYS_I2S_RST  / E_SYS_PDMA_RST   
 * @return     None 
************************************************************************************************************/
void DrvSYS_ResetIP(E_SYS_IP_RST eIpRst)
{   
    uint32_t u32Reg;

    if (eIpRst == E_SYS_PDMA_RST)
    {
        REG32WRITE(SYS->IPRSTC1, PDMA_RST, 1);
        REG32WRITE(SYS->IPRSTC1, PDMA_RST, 0);
    }
    else
    {
        u32Reg = *((__IO uint32_t *) &SYS->IPRSTC2);
        *((__IO uint32_t *) &SYS->IPRSTC2) = u32Reg | (1<<eIpRst);
        *((__IO uint32_t *) &SYS->IPRSTC2) = u32Reg & ~(1<<eIpRst);
    }
}

/********************************************************************************************************//**
 * @brief     The function is used to Reset CPU 
 * @param[in] None
 * @return     None 
************************************************************************************************************/
void DrvSYS_ResetCPU(void)
{
    REG32WRITE(SYS->IPRSTC1, CPU_RST, 1);
}


/********************************************************************************************************//**
 * @brief     The function is used to Reset whole chip
 * @param[in] None
 * @return     None 
************************************************************************************************************/
void DrvSYS_ResetChip(void)
{
    REG32WRITE(SYS->IPRSTC1, CHIP_RST, 1);
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable BOD function 
 * @param[in] i32Enable   1 : Enable / 0 : Disable
 * @return     None 
************************************************************************************************************/
void DrvSYS_EnableBOD(int32_t i32Enable)
{
    REG32WRITE(SYS->BODCR, BOD_EN, i32Enable);
}
     
 
/********************************************************************************************************//**
 * @brief     The function is used to Select BOD threshold voltage
 * @param[in] u8Volt      3:4.5V, 2:3.8V, 1:2.6V, 0:2.2V
 * @return     None 
************************************************************************************************************/
void DrvSYS_SelectBODVolt(uint8_t u8Volt)
{
    REG32WRITE(SYS->BODCR, BOD_VL, u8Volt);
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable BOD reset function or interrupt function, and install call back function if         
 *               interrupt function is enabled.   
 * @param[in] i32Enable  1: Enable BOD reset function / 0:Enable BOD interrupt function
 * @param[in]  bodcallbackFn :  BOD Call back function      
 * @return     None 
************************************************************************************************************/
void DrvSYS_EnableBODRst(int32_t i32Enable, BOD_CALLBACK bodcallbackFn)
{
    REG32WRITE(SYS->BODCR, BOD_RSTEN, i32Enable);
    
    if (i32Enable)
    {       
        NVIC_DisableIRQ(BOD_IRQn);
    }
    else
    { 
        BOD_callbackFn = bodcallbackFn;
        NVIC_EnableIRQ(BOD_IRQn);
    }   
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable BOD low power mode
 * @param[in] i32Enable   1 : Enable / 0 : Disable  
 * @return     None 
************************************************************************************************************/
void DrvSYS_EnableBODLowPowerMode(int32_t i32Enable)
{
    REG32WRITE(SYS->BODCR, BOD_LPM, i32Enable);
}


/********************************************************************************************************//**
 * @brief     The function is used to Get BOD state
 * @param[in] None
 * @return    BOD output status  - 1: the detected voltage is lower than BOD threshold voltage
 *                               - 0: the detected voltage is higher than BOD threshold voltage 
************************************************************************************************************/
uint32_t DrvSYS_GetBODState(void)
{
    return SYS->BODCR.BOD_OUT;
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable low voltage reset 
 * @param[in] i32Enable    1 : Enable / 0 : Disable
 * @return     None 
************************************************************************************************************/
void DrvSYS_EnableLowVoltRst(int32_t i32Enable)
{
    REG32WRITE(SYS->BODCR, LVR_EN, i32Enable);
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable Tempature Sensor
 * @param[in] i32Enable    1 : Enable / 0 : Disable
 * @return     None 
************************************************************************************************************/
void DrvSYS_EnableTempatureSensor(int32_t i32Enable)
{
    SYS->TEMPCR = i32Enable;
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Set RC adjustment value
 * @param[in] u32Adj : RC adjustment value 
 * @return     None 
************************************************************************************************************/
void DrvSYS_SetRCAdjValue(uint32_t u32Adj)
{
    SYS->RCADJ = u32Adj;
}
 
/********************************************************************************************************//**
 * @brief     The function is used to ISR to handle BOD interrupt event
 * @param[in]  None
 * @return     None 
************************************************************************************************************/
void BOD_IRQHandler(void)
{
    REG32WRITE(SYS->BODCR, BOD_INTF, 1);
    if (BOD_callbackFn != NULL)
        BOD_callbackFn();

}


/********************************************************************************************************//**
 * @brief     The function is used to Enable/Disable coresponding IP clock
 * @param[in]  eIpClk               E_SYS_WDG_CLK   / E_SYS_RTC_CLK   / E_SYS_TMR0_CLK  / E_SYS_TMR1_CLK    
 *                                  E_SYS_TMR2_CLK  / E_SYS_TMR3_CLK  / E_SYS_FDIV_CLK  / E_SYS_I2C0_CLK   
 *                                  E_SYS_I2C1_CLK  / E_SYS_SPI0_CLK  / E_SYS_SPI1_CLK  / E_SYS_SPI2_CLK    
 *                                  E_SYS_SPI3_CLK  / E_SYS_UART0_CLK / E_SYS_UART1_CLK / E_SYS_UART2_CLK   
 *                                  E_SYS_PWM01_CLK / E_SYS_PWM23_CLK / E_SYS_PWM45_CLK / E_SYS_PWM67_CLK   
 *                                  E_SYS_CAN0_CLK  / E_SYS_USBD_CLK  / E_SYS_ADC_CLK   / E_SYS_I2S_CLK     
 *                                  E_SYS_ACMP_CLK  / E_SYS_PS2_CLK   / E_SYS_PDMA_CLK  / E_SYS_ISP_CLK  
 * @param[in]  i32Enable        1 : Enable / 0 : Disable  
 * @return     None 
************************************************************************************************************/
void DrvSYS_SetIPClock(E_SYS_IP_CLK eIpClk, int32_t i32Enable)
{   
    uint32_t u32Reg;

    if (eIpClk == E_SYS_PDMA_CLK)
    {
        REG32WRITE(SYSCLK->AHBCLK, PDMA_EN, i32Enable);
    }
    else if (eIpClk == E_SYS_ISP_CLK)
    {
        REG32WRITE(SYSCLK->AHBCLK, ISP_EN, i32Enable);
    }
    else
    {
        u32Reg = *((__IO uint32_t *) &SYSCLK->APBCLK);
        
        if (i32Enable)
        {
            *((__IO uint32_t *) &SYSCLK->APBCLK) = u32Reg | (1<<eIpClk);
        }
        else
        {
            *((__IO uint32_t *) &SYSCLK->APBCLK) = u32Reg & ~(1<<eIpClk);
        }
    }
}

 
/********************************************************************************************************//**
 * @brief     The function is used to select HCLK clock source
 * @param[in]  u8ClkSrcSel              0: External 12M clock                                            
 *                                      1: External 32K clock                                             
 *                                      2: PLL clock                                                       
 *                                      3: Internal 10K clock                                               
 *                                      7: Internal 22M clock   
 * @return     - 0:  Success    
 *             - < 0:  Wrong value 
************************************************************************************************************/
int32_t DrvSYS_SetHCLKSource(uint8_t u8ClkSrcSel)
{
    if ((u8ClkSrcSel > 7) || ((u8ClkSrcSel > 3) && (u8ClkSrcSel < 7)))
    {
        return E_DRVSYS_ERR_ARGUMENT;
    }
    else
    {
        REG32WRITE(SYSCLK->CLKSEL0, HCLK_S, u8ClkSrcSel);
    }
    return 0;
}


/********************************************************************************************************//**
 * @brief     The function is used to select sysyem tick clock source
 * @param[in]  u8ClkSrcSel              0: External 12M clock                                              
 *                                      1: External 32K clock                                             
 *                                      2: External 12M clock / 2                                         
 *                                      3: HCLK / 2                                                         
 *                                      4~7: Internal 22M clock / 2   
 * @return     - 0:  Success    
 *             - < 0:  Wrong value 
************************************************************************************************************/
int32_t DrvSYS_SetSysTickSource(uint8_t u8ClkSrcSel)
{
    if (u8ClkSrcSel > 7)
    {
        return E_DRVSYS_ERR_ARGUMENT;
    }
    else
    {
        REG32WRITE(SYSCLK->CLKSEL0, STCLK_S, u8ClkSrcSel);
    }
    return 0;
}

 
/********************************************************************************************************//**
 * @brief     This function is used to select IP clock source
 * @param[in]  eIpClkSrc                E_SYS_WDG_CLKSRC   / E_SYS_ADC_CLKSRC  / E_SYS_TMR0_CLKSRC    
 *                                      E_SYS_TMR1_CLKSRC  / E_SYS_TMR2_CLKSRC / E_SYS_TMR3_CLKSRC       
 *                                      E_SYS_UART_CLKSRC  / E_SYS_CAN_CLKSRC  / E_SYS_PWM01_CLKSRC        
 *                                      E_SYS_PWM23_CLKSRC / E_SYS_PWM45_CLKSRC/ E_SYS_PWM67_CLKSRC         
 *                                      E_SYS_FRQDIV_CLKSRC/ E_SYS_I2S_CLKSRC 
 * @param[in]   u8ClkSrcSel  Corresponding clock source       
 * @return     - 0:  Success    
 *             - < 0:  Wrong value 
************************************************************************************************************/
int32_t DrvSYS_SetIPClockSource(E_SYS_IP_CLKSRC eIpClkSrc, uint8_t u8ClkSrcSel)
{
    
    switch(eIpClkSrc)
    {
        case E_SYS_FRQDIV_CLKSRC:  
            REG32WRITE(SYSCLK->CLKSEL2, FRQDIV_S, u8ClkSrcSel);
            break;
        case E_SYS_I2S_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL2, I2S_S, u8ClkSrcSel);
            break;
        case E_SYS_PWM67_CLKSRC:  
            REG32WRITE(SYSCLK->CLKSEL2, PWM67_S, u8ClkSrcSel);
            break;
        case E_SYS_PWM45_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL2, PWM45_S, u8ClkSrcSel);
            break;
        case E_SYS_PWM23_CLKSRC:  
            REG32WRITE(SYSCLK->CLKSEL1, PWM23_S, u8ClkSrcSel);
            break;
        case E_SYS_PWM01_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, PWM01_S, u8ClkSrcSel);
            break;
        case E_SYS_CAN_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, CAN_S, u8ClkSrcSel);
            break;
        case E_SYS_UART_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, UART_S, u8ClkSrcSel);
            break;
        case E_SYS_TMR3_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, TMR3_S, u8ClkSrcSel);
            break;              
        case E_SYS_TMR2_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, TMR2_S, u8ClkSrcSel);
            break;              
        case E_SYS_TMR1_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, TMR1_S, u8ClkSrcSel);
            break;              
        case E_SYS_TMR0_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, TMR0_S, u8ClkSrcSel);
            break;              
        case E_SYS_ADC_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, ADC_S, u8ClkSrcSel);
            break;              
        case E_SYS_WDG_CLKSRC:
            REG32WRITE(SYSCLK->CLKSEL1, WDG_S, u8ClkSrcSel);
            break;              
        default:
            return E_DRVSYS_ERR_IPSRC;
    }
    
    return E_SUCCESS;
}

 
/********************************************************************************************************//**
 * @brief      This function is used to set IP divider number from the corresponding clock source   
 * @param[in]  eIpDiv   E_SYS_ADC_DIV / E_SYS_CAN_DIV / E_SYS_UART_DIV / E_SYS_USB_DIV /E_SYS_HCLK_DIV                                         
 * @param[in]  i32value   0 ~ 255 for ADC, 0 ~ 15 for other IP or HCLK      
 * @return     - 0:  Success    
 *             - < 0:  Wrong value 
************************************************************************************************************/
int32_t DrvSYS_SetClockDivider(E_SYS_IP_DIV eIpDiv , int32_t i32value)
{
    if (eIpDiv == E_SYS_ADC_DIV)
    {
        REG32WRITE(SYSCLK->CLKDIV, ADC_N, i32value);
    }
    else if(eIpDiv == E_SYS_CAN_DIV)
    {
        REG32WRITE(SYSCLK->CLKDIV, CAN_N, i32value);
    }
    else if(eIpDiv == E_SYS_UART_DIV)
    {
        REG32WRITE(SYSCLK->CLKDIV, UART_N, i32value);
    }
    else if(eIpDiv == E_SYS_USB_DIV)
    {
        REG32WRITE(SYSCLK->CLKDIV, USB_N, i32value);
    }
    else if(eIpDiv == E_SYS_HCLK_DIV)
    {
        REG32WRITE(SYSCLK->CLKDIV, HCLK_N, i32value);
    }
    else
        return E_DRVSYS_ERR_IPDIV;

    return E_SUCCESS;
    
}


 
/********************************************************************************************************//**
 * @brief     This function is used to enable/disable Oscillator    
 * @param[in]  eOscCtrl      E_SYS_XTL12M  / E_SYS_XTL32K  / E_SYS_OSC22M / E_SYS_OSC10K                                               
 * @param[in]  i32Enable     1: Enable / 0: Disable 
 * @return     - 0:  Success    
 *             - < 0:  Wrong value 
************************************************************************************************************/
int32_t DrvSYS_SetOscCtrl(E_SYS_OSC_CTRL eOscCtrl, int32_t i32Enable)
{   
    switch(eOscCtrl)
    {
        case E_SYS_XTL12M:  
            REG32WRITE(SYSCLK->PWRCON, XTL12M_EN, i32Enable);
            break;
        case E_SYS_XTL32K:
            REG32WRITE(SYSCLK->PWRCON, XTL32K_EN, i32Enable);
            break;
        case E_SYS_OSC22M:
            REG32WRITE(SYSCLK->PWRCON, OSC22M_EN, i32Enable);
            break;
        case E_SYS_OSC10K:
            REG32WRITE(SYSCLK->PWRCON, OSC10K_EN, i32Enable);
            break;              
        default:
            return E_DRVSYS_ERR_ARGUMENT;
    }
    
    return E_SUCCESS;
}

 
/********************************************************************************************************//**
 * @brief     The function is used to ISR to handle Power Down Wake up interrupt event 
 * @param[in]   None                                            
 * @return     None     
************************************************************************************************************/
void PWRWU_IRQHandler(void)
{
    REG32WRITE(SYSCLK->PWRCON, PD_WU_STS, 1);
    if (PWRWU_callbackFn != NULL)
        PWRWU_callbackFn(); 
}

 
/********************************************************************************************************//**
 * @brief     The function is used to  Enable or disable PD_WU interrupt function, and install call back function, and         
 *               enable 64 clock cycle delay to wait oscillator clock stable 
 * @param[in]  i32Enable                1: Enable Power dowm Wake up interrup                              
 *                                      0: Disable Power dowm Wake up interrup  
 * @param[in]  pdwucallbackFn           Power Down Wake Up Call back function  
 * @param[in]  i32enWUDelay             1: Enable 64 clock cycle delay                                     
 *                                      0: Disable 64 clock cycle delay 
 * @return     None     
************************************************************************************************************/
void DrvSYS_EnablePWRWUInt(int32_t i32Enable, PWRWU_CALLBACK pdwucallbackFn, int32_t i32enWUDelay)
{
    REG32WRITE(SYSCLK->PWRCON, WINT_EN, i32Enable);
    
    if (i32Enable)
    {       
        REG32WRITE(SYSCLK->PWRCON, WU_DLY, i32enWUDelay);
        PWRWU_callbackFn = pdwucallbackFn;
        NVIC_EnableIRQ(PWRWU_IRQn);
    }
    else
    { 
        REG32WRITE(SYSCLK->PWRCON, WU_DLY, 0);
        PWRWU_callbackFn = NULL;
        NVIC_DisableIRQ(PWRWU_IRQn);
    }   
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable or active power down function 
 * @param[in]  i32Enable     1: Enable / 0: Disable                                             
 * @return     None     
************************************************************************************************************/
void DrvSYS_EnablePowerDown(int32_t i32Enable)
{
    REG32WRITE(SYSCLK->PWRCON, PWR_DOWN, i32Enable);
}

 
/********************************************************************************************************//**
 * @brief     This function is used to control power down entry condition 
 * @param[in]  i32Enable    - [in]  1: Enter power down mode when PWR_DOWN and CPU runs WFE/WFI instruction 
 *                                  0: Enter power down mode when PWR_DOWN is set                                           
 * @return     None     
************************************************************************************************************/
void DrvSYS_SetPowerDownWaitCPU(int32_t i32Enable)
{
    REG32WRITE(SYSCLK->PWRCON, PD_WAIT_CPU, i32Enable);
}


 
/********************************************************************************************************//**
 * @brief     This function is used to select PLL clock source  
 * @param[in]  u8ClkSrcSel              E_DRVSYS_EXT_12M: External 12M clock                             
 *                                      E_DRVSYS_INT_22M: Internal 22M clock                                            
 * @return     None     
************************************************************************************************************/
void DrvSYS_SetPllSrc(E_DRVSYS_SRC_CLK ePllSrc)
{
    REG32WRITE(SYSCLK->PLLCON, PLL_SRC, ePllSrc);
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Enable PLL power down function 
 * @param[in]  i32Enable       1: Power down mode / 0: Normal mode                                              
 * @return     None     
************************************************************************************************************/
void DrvSYS_SetPLLPowerDown(int32_t i32Enable)
{
    REG32WRITE(SYSCLK->PLLCON, OE, i32Enable);
    REG32WRITE(SYSCLK->PLLCON, PD, i32Enable);
}


 
/********************************************************************************************************//**
 * @brief     The function is used to Get external clock frequency.
 * @param[in]   None                                            
 * @return     The external clock frequency     
************************************************************************************************************/
uint32_t DrvSYS_GetEXTClock(void)
{   
    return s_u32ExtClockKHz;
}


 
/********************************************************************************************************//**
 * @brief     This function is used to calculate the nearest PLL frequency to fit the target PLL frequency      
 *            which is defined by u32PllClockKHz and return the relative PLL control register setting.  
 * @param[in]  u32ExtClockKHz   The external clock frequency (kHz)   
 * @param[in]  u32PllClockKHz   The target PLL clock frequency (kHz) 
 * @return     The PLL register setting for the target PLL clock frequency.     
************************************************************************************************************/
uint32_t DrvSYS_GetPllContent(uint32_t u32ExtClockKHz, uint32_t u32PllClockKHz)
{
    uint32_t u32NR, u32NF;
    uint32_t u32Register=0;

    u32PllClockKHz = u32PllClockKHz << 2;
    
    u32NF = u32PllClockKHz / 1000;
    u32NR = u32ExtClockKHz / 1000;
   
    if (SYSCLK->PLLCON.PLL_SRC == 0)
    {
        u32NR >>= 2;
        u32NF >>= 2;    
    }
    else
    {
        u32NR >>= 1;
        u32NF >>= 1;    
    }
        
    u32Register = (0x03<<14)| ((u32NR - 2)<<9) | (u32NF - 2);

    /* Return the nearest value */
    return u32Register;
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Get PLL clock frquency. The clock UNIT is in kHz 
 * @param[in]   None                                            
 * @return     The PLL clock
************************************************************************************************************/
uint32_t DrvSYS_GetPLLClock(void)
{
    uint32_t u32Freq =0, u32PLLSrc;
    uint32_t au32Map[4] = {1, 2, 2, 4};

    if (SYSCLK->PLLCON.PLL_SRC == 1)
    {
        u32PLLSrc = 22000;  
    }
    else
    {
        u32PLLSrc = s_u32ExtClockKHz;
    }

    if (SYSCLK->PLLCON.PD ==1)  
        u32Freq = 0;    
    else        
        u32Freq = u32PLLSrc * (SYSCLK->PLLCON.FB_DV+2) / (SYSCLK->PLLCON.IN_DV+2) / au32Map[SYSCLK->PLLCON.OUT_DV];

    return u32Freq;
}

 
/********************************************************************************************************//**
 * @brief     The function is used to Get HCLK clock and the UNIT is in kHz
 * @param[in]   None                                            
 * @return    HCLK clock frequency in kHz   
************************************************************************************************************/
uint32_t DrvSYS_GetHCLK(void)
{
    
    uint32_t u32Freqout = 0, u32AHBDivider;

    if (SYSCLK->CLKSEL0.HCLK_S == 0)        /* external 12MHz crystal clock */
    {
        u32Freqout = s_u32ExtClockKHz;  
    }
    else if(SYSCLK->CLKSEL0.HCLK_S == 1)    /* external 32KHz crystal clock */ 
    {
        u32Freqout = 32;
    }
    else if(SYSCLK->CLKSEL0.HCLK_S == 2)    /* PLL clock */
    {
        u32Freqout = DrvSYS_GetPLLClock();
    }
    else if(SYSCLK->CLKSEL0.HCLK_S == 3)    /* internal 10KHz oscillator clock */
    {
        u32Freqout = 10;
    }
    else                                    /* internal 22MHz oscillator clock */
    {
        u32Freqout = 22000;
    
    }
    u32AHBDivider = SYSCLK->CLKDIV.HCLK_N + 1 ;
    
    return (u32Freqout/u32AHBDivider);  
}


/********************************************************************************************************//**
 * @brief     The function is used to configure the PLL according to the external clock and target PLL clock
 * @param[in]   u32ExtClockKHz   The external clock frequency in kHz   
 * @param[in]   u32PllClockKHz   The target PLL clock frequency in kHz
 * @return     E_SUCCESS    
************************************************************************************************************/
int32_t DrvSYS_Open(uint32_t u32ExtClockKHz, uint32_t u32PllClockKHz)
{
    uint32_t u32PllCr;  
    volatile uint32_t delayCnt;
    uint32_t isOsc22mEn;
    
    s_u32ExtClockKHz = u32ExtClockKHz;  
    u32PllCr = DrvSYS_GetPllContent(u32ExtClockKHz, u32PllClockKHz);

    /* Backup internal 22MHz clock status */
    isOsc22mEn = SYSCLK->PWRCON.OSC22M_EN;

    /* Force to internal 22MHz clock */
    REG32WRITE(SYSCLK->PWRCON,OSC22M_EN , 1);
    DrvSYS_SetHCLKSource(4);
    
    REG32WRITE(SYSCLK->PLLCON, FB_DV, (u32PllCr & 0x01FF));
    REG32WRITE(SYSCLK->PLLCON, IN_DV, (u32PllCr >> 9) & 0x1F);
    REG32WRITE(SYSCLK->PLLCON, OUT_DV, (u32PllCr >> 14) & 0x03);
    REG32WRITE(SYSCLK->PLLCON, OE, 0);
    REG32WRITE(SYSCLK->PLLCON, PD, 0);
    
    /*Delay for PLL stable*/
    for (delayCnt=0;delayCnt<10000;delayCnt++);

    /* Change HCLK clock source to be PLL. */
    DrvSYS_SetHCLKSource(2);

    /* Restore internal 22MHz clock source status */ 
    REG32WRITE(SYSCLK->PWRCON, OSC22M_EN, isOsc22mEn);
    
    return E_SUCCESS;

}


/********************************************************************************************************//**
 * @brief     The function is used to Enable frequency clock output and set its divider value
 * @param[in]   i32Enable         1: Enable / 0: Disable    
 * @param[in]   u8Divider         The divider of output frequency (0~15)     
 * @return      - 0:  Success       
 *              - < 0:  Wrong value      
************************************************************************************************************/
int32_t DrvSYS_EnableFreqDividerOutput(int32_t i32Enable, uint8_t u8Divider)
{
    if (u8Divider > 15)
        return E_DRVSYS_ERR_ARGUMENT;

    if (i32Enable)
    {
        REG32WRITE(SYSCLK->FRQDIV, FSEL, u8Divider);
    }
    REG32WRITE(SYSCLK->FRQDIV, FDIV_EN, i32Enable);         
    DrvSYS_SetIPClock(E_SYS_FDIV_CLK, i32Enable);

    return E_SUCCESS;
}

